@@ -7,11 +7,15 @@ def is_(x, y):
 
 def test_no_ensure_ascii():
     assert is_(json.dumps(u"\u1234", ensure_ascii=False), u'"\u1234"')
-    assert is_(json.dumps(u"\xc0", ensure_ascii=False), u'"\xc0"')
-    with pytest.raises(TypeError):
-        json.dumps((u"\u1234", b"x"), ensure_ascii=False)
-    with pytest.raises(TypeError):
-        json.dumps((b"x", u"\u1234"), ensure_ascii=False)
+    assert is_(json.dumps("\xc0", ensure_ascii=False), '"\xc0"')
+    with pytest.raises(UnicodeDecodeError) as excinfo:
+        json.dumps((u"\u1234", "\xc0"), ensure_ascii=False)
+    assert str(excinfo.value).startswith(
+        "'ascii' codec can't decode byte 0xc0 ")
+    with pytest.raises(UnicodeDecodeError) as excinfo:
+        json.dumps(("\xc0", u"\u1234"), ensure_ascii=False)
+    assert str(excinfo.value).startswith(
+        "'ascii' codec can't decode byte 0xc0 ")
 
 def test_issue2191():
     assert is_(json.dumps(u"xxx", ensure_ascii=False), u'"xxx"')
