@@ -8,13 +8,13 @@ class StringOrUnicodeBuilder(object):
     def __init__(self):
         self._builder = StringBuilder()
     def append(self, string):
-        try:
-            self._builder.append(string)
-        except UnicodeEncodeError:
+        if (isinstance(string, unicode) and
+                type(self._builder) is StringBuilder):
             ub = UnicodeBuilder()
             ub.append(self._builder.build())
             self._builder = ub
-            ub.append(string)
+            self.append = ub.append   # shortcut only
+        self._builder.append(string)
     def build(self):
         return self._builder.build()
 
@@ -35,7 +35,7 @@ for i in range(0x20):
     ESCAPE_DCT.setdefault(chr(i), '\\u%04x' % (i,))
 
 INFINITY = float('inf')
-FLOAT_REPR = repr
+FLOAT_REPR = float.__repr__
 
 def raw_encode_basestring(s):
     """Return a JSON representation of a Python string
@@ -529,3 +529,10 @@ class JSONEncoder(object):
                                           _current_indent_level):
                 yield chunk
             self.__remove_markers(markers, o)
+
+
+# overwrite some helpers here with more efficient versions
+try:
+    from _pypyjson import raw_encode_basestring_ascii
+except ImportError:
+    pass
