@@ -4,8 +4,6 @@ import _rawffi
 from _ctypes.basics import _CData, cdata_from_address, _CDataMeta, sizeof
 from _ctypes.basics import keepalive_key, store_reference, ensure_objects
 from _ctypes.basics import CArgObject, as_ffi_pointer
-from _pypy_generic_alias import GenericAlias
-import sys, __pypy__, struct
 
 class ArrayMeta(_CDataMeta):
     def __new__(self, name, cls, typedict):
@@ -14,14 +12,10 @@ class ArrayMeta(_CDataMeta):
         if cls == (_CData,): # this is the Array class defined below
             res._ffiarray = None
             return res
-        if not hasattr(res, '_length_'):
+        if not hasattr(res, '_length_') or not isinstance(res._length_, int):
             raise AttributeError(
                 "class must define a '_length_' attribute, "
                 "which must be a positive integer")
-        if not isinstance(res._length_, int):
-            raise TypeError("The '_length_' attribute must be an integer")
-        if res._length_ < 0:
-            raise ValueError("The '_length_' attribute must not be negative")
         ffiarray = res._ffiarray = _rawffi.Array(res._type_._ffishape_)
         subletter = getattr(res._type_, '_type_', None)
         if subletter == 'c':
@@ -59,7 +53,7 @@ class ArrayMeta(_CDataMeta):
                 # we don't want to have buffers here
                 if len(val) > self._length_:
                     raise ValueError("%r too long" % (val,))
-                if isinstance(val, str):
+                if isinstance(val, unicode):
                     target = self._buffer
                 else:
                     target = self
@@ -87,11 +81,8 @@ class ArrayMeta(_CDataMeta):
     def _CData_output(self, resarray, base=None, index=-1):
         from _rawffi.alt import types
         # If a char_p or unichar_p is received, skip the string interpretation
-        try:
-            deref = type(base)._deref_ffiargtype()
-        except AttributeError:
-            deref = None
-        if deref != types.char_p and deref != types.unichar_p:
+        if base._ffiargtype != types.Pointer(types.char_p) and \
+           base._ffiargtype != types.Pointer(types.unichar_p):
             # this seems to be a string if we're array of char, surprise!
             from ctypes import c_char, c_wchar
             if self._type_ is c_char:
@@ -113,46 +104,21 @@ class ArrayMeta(_CDataMeta):
         # array accepts very strange parameters as part of structure
         # or function argument...
         from ctypes import c_char, c_wchar
-        if isinstance(value, self):
-            return value
-        if hasattr(self, '_type_'):
-            if issubclass(self._type_, c_char):
-                if isinstance(value, bytes):
-                    if len(value) > self._length_:
-                        raise ValueError("Invalid length")
-                    value = self(*value)
-                elif not isinstance(value, self):
-                    raise TypeError("expected bytes, %s found"
-                                    % (value.__class__.__name__,))
-            elif issubclass(self._type_, c_wchar):
-                if isinstance(value, str):
-                    if len(value) > self._length_:
-                        raise ValueError("Invalid length")
-                    value = self(*value)
-                elif not isinstance(value, self):
-                    raise TypeError("expected unicode string, %s found"
-                                    % (value.__class__.__name__,))
-        if isinstance(value, tuple):
-            if len(value) > self._length_:
-                raise RuntimeError("Invalid length")
-            value = self(*value)
+        if issubclass(self._type_, (c_char, c_wchar)):
+            if isinstance(value, basestring):
+                if len(value) > self._length_:
+                    raise ValueError("Invalid length")
+                value = self(*value)
+            elif not isinstance(value, self):
+                raise TypeError("expected string or Unicode object, %s found"
+                                % (value.__class__.__name__,))
+        else:
+            if isinstance(value, tuple):
+                if len(value) > self._length_:
+                    raise RuntimeError("Invalid length")
+                value = self(*value)
         return _CDataMeta.from_param(self, value)
 
-    def _build_ffiargtype(self):
-        return _ffi.types.Pointer(self._type_.get_ffi_argtype())
-
-    def _deref_ffiargtype(self):
-        return self._type_.get_ffi_argtype()
-
-    def _getformat(self):
-        shape = []
-        tp = self
-        while hasattr(tp, '_length_'):
-            shape.append(tp._length_)
-            tp = tp._type_
-        return "(%s)%s" % (','.join([str(n) for n in shape]), tp._getformat())
-
-
 def array_get_slice_params(self, index):
     if hasattr(self, '_length_'):
         start, stop, step = index.indices(self._length_)
@@ -198,7 +164,8 @@ def array_slice_getitem(self, index):
         return u"".join(l)
     return l
 
-class Array(_CData, metaclass=ArrayMeta):
+class Array(_CData):
+    __metaclass__ = ArrayMeta
     _ffiargshape_ = 'P'
 
     def __init__(self, *args):
@@ -206,7 +173,6 @@ class Array(_CData, metaclass=ArrayMeta):
             self._buffer = self._ffiarray(self._length_, autofree=True)
         for i, arg in enumerate(args):
             self[i] = arg
-    _init_no_arg_ = __init__
 
     def _fix_index(self, index):
         if index < 0:
@@ -264,34 +230,13 @@ class Array(_CData, metaclass=ArrayMeta):
     def _as_ffi_pointer_(self, ffitype):
         return as_ffi_pointer(self, ffitype)
 
-    def __buffer__(self, flags):
-        shape = []
-        obj = self
-        while 1:
-            shape.append(obj._length_)
-            try:
-                obj[0]._length_
-            except (AttributeError, IndexError):
-                break
-            obj = obj[0]
-
-        fmt = obj._type_._getformat()
-        itemsize = sizeof(obj._type_)
-        return __pypy__.newmemoryview(memoryview(self._buffer), itemsize, fmt, shape)
-
-    def __class_getitem__(self, item):
-        return GenericAlias(self, item)
-
 ARRAY_CACHE = {}
 
 def create_array_type(base, length):
-    if not hasattr(length, '__index__'):
-        raise TypeError("Can't multiply a ctypes type by a non-int of type %s" % type(length).__name__)
-    length = int(length)
+    if not isinstance(length, (int, long)):
+        raise TypeError("Can't multiply a ctypes type by a non-integer")
     if length < 0:
         raise ValueError("Array length must be >= 0")
-    if length * base._sizeofinstances() > sys.maxsize:
-        raise OverflowError("array too large")
     key = (base, length)
     try:
         return ARRAY_CACHE[key]
@@ -302,11 +247,6 @@ def create_array_type(base, length):
             _type_ = base
         )
         cls = ArrayMeta(name, (Array,), tpdict)
+        cls._ffiargtype = _ffi.types.Pointer(base.get_ffi_argtype())
         ARRAY_CACHE[key] = cls
         return cls
-
-byteorder = {'little': '<', 'big': '>'}
-swappedorder = {'little': '>', 'big': '<'}
-
-def get_format_str(typ):
-    return typ._getformat()
