@@ -1,15 +1,7 @@
 import sys
-import __pypy__
 import _continuation
 
-__version__ = "0.4.13"
-
-# ____________________________________________________________
-# Constants from greenlet 1.0.0
-
-GREENLET_USE_GC = True
-GREENLET_USE_TRACING = True
-GREENLET_USE_CONTEXT_VARS = False
+__version__ = "0.4.12"
 
 # ____________________________________________________________
 # Exceptions
@@ -70,16 +62,6 @@ class greenlet(_continulet):
         while not (target.__main or _continulet.is_pending(target)):
             # inlined __nonzero__ ^^^ in case it's overridden
             if not target.__started:
-                # check that 'target.parent' runs in the current thread,
-                # at least.  It can be changed arbitrarily afterwards in
-                # pypy greenlets, but too bad
-                parent1 = target.parent
-                while not parent1.__started:
-                    parent1 = parent1.parent
-                if parent1.__thread_id is not _tls.thread_id:
-                    raise error("cannot start greenlet because its 'parent'"
-                                " is running on a different thread")
-
                 if methodname == 'switch':
                     greenlet_func = _greenlet_start
                 else:
@@ -87,7 +69,6 @@ class greenlet(_continulet):
                 _continulet.__init__(target, greenlet_func, *baseargs)
                 methodname = 'switch'
                 baseargs = ()
-                target.__thread_id = _tls.thread_id
                 target.__started = True
                 break
             # already done, go to the parent instead
@@ -98,16 +79,12 @@ class greenlet(_continulet):
             # convert a "raise GreenletExit" into "return GreenletExit"
             if methodname == 'throw':
                 try:
-                    raise __pypy__.normalize_exc(baseargs[0], baseargs[1])
-                except GreenletExit as e:
+                    raise baseargs[0], baseargs[1]
+                except GreenletExit, e:
                     methodname = 'switch'
                     baseargs = (((e,), {}),)
                 except:
                     baseargs = sys.exc_info()[:2] + baseargs[2:]
-        else:
-            if target.__thread_id is not _tls.thread_id:
-                raise error("cannot switch to greenlet running in a"
-                            " different thread")
         #
         try:
             unbound_method = getattr(_continulet, methodname)
@@ -134,7 +111,7 @@ class greenlet(_continulet):
         else:
             return args
 
-    def __bool__(self):
+    def __nonzero__(self):
         return self.__main or _continulet.is_pending(self)
 
     @property
@@ -149,7 +126,7 @@ class greenlet(_continulet):
             return None
         if self.__main:
             self = getcurrent()
-        f = self._get_frame()
+        f = _continulet.__reduce__(self)[2][0]
         if not f:
             return None
         return f.f_back.f_back.f_back   # go past start(), __switch(), switch()
@@ -196,9 +173,7 @@ _tls = _local()
 def _green_create_main():
     # create the main greenlet for this thread
     _tls.current = None
-    _tls.thread_id = object()
     gmain = greenlet.__new__(greenlet)
-    gmain._greenlet__thread_id = _tls.thread_id
     gmain._greenlet__main = True
     gmain._greenlet__started = True
     assert gmain.parent is None
@@ -213,7 +188,7 @@ def _greenlet_start(greenlet, args):
             if hasattr(_tls, 'trace'):
                 _run_trace_callback('switch')
             res = greenlet.run(*args, **kwds)
-        except GreenletExit as e:
+        except GreenletExit, e:
             res = e
         finally:
             _continuation.permute(greenlet, greenlet.parent)
@@ -227,8 +202,8 @@ def _greenlet_throw(greenlet, exc, value, tb):
         try:
             if hasattr(_tls, 'trace'):
                 _run_trace_callback('throw')
-            raise __pypy__.normalize_exc(exc, value, tb)
-        except GreenletExit as e:
+            raise exc, value, tb
+        except GreenletExit, e:
             res = e
         finally:
             _continuation.permute(greenlet, greenlet.parent)
