@@ -1,6 +1,4 @@
-from rpython.rlib.rstruct.error import StructError
-from rpython.rlib.buffer import StringBuffer, SubBuffer, RawBuffer
-from rpython.rlib.mutbuffer import MutableStringBuffer
+from rpython.rlib.buffer import StringBuffer, SubBuffer
 
 from pypy.interpreter.error import oefmt
 
@@ -10,7 +8,7 @@ class BufferInterfaceNotFound(Exception):
 
 class BufferView(object):
     """Abstract base class for buffers."""
-    _attrs_ = ['readonly', 'w_obj']
+    _attrs_ = ['readonly']
     _immutable_ = True
 
     def getlength(self):
@@ -70,19 +68,6 @@ class BufferView(object):
         fmtiter.interpret(self.getformat())
         return fmtiter.result_w[0]
 
-    def bytes_from_value(self, space, w_val):
-        from pypy.module.struct.formatiterator import PackFormatIterator
-        itemsize = self.getitemsize()
-        buf = MutableStringBuffer(itemsize)
-        fmtiter = PackFormatIterator(space, buf, [w_val])
-        try:
-            fmtiter.interpret(self.getformat())
-        except StructError as e:
-            raise oefmt(space.w_TypeError,
-                        "memoryview: invalid type for format '%s'",
-                        self.getformat())
-        return buf.finish()
-
     def _copy_buffer(self):
         if self.getndim() == 0:
             itemsize = self.getitemsize()
@@ -138,16 +123,10 @@ class BufferView(object):
         itemsize = self.getitemsize()
         # TODO: this probably isn't very fast
         data = self.getbytes(offset, itemsize)
-        return self.value_from_bytes(space, data)
+        return space.newbytes(data)
 
     def new_slice(self, start, step, slicelength):
-        return BufferSlice(self, start, step, slicelength, w_obj=self.w_obj)
-
-    def setitem_w(self, space, idx, w_obj):
-        offset = self.get_offset(space, 0, idx)
-        # TODO: this probably isn't very fast
-        byteval = self.bytes_from_value(space, w_obj)
-        self.setbytes(offset, byteval)
+        return BufferSlice(self, start, step, slicelength)
 
     def w_tolist(self, space):
         dim = self.getndim()
@@ -155,7 +134,7 @@ class BufferView(object):
             raise NotImplementedError
         elif dim == 1:
             n = self.getshape()[0]
-            values_w = [self.w_getitem(space, i) for i in range(n)]
+            values_w = [space.ord(self.w_getitem(space, i)) for i in range(n)]
             return space.newlist(values_w)
         else:
             return self._tolist_rec(space, 0, 0)
@@ -188,11 +167,14 @@ class BufferView(object):
         return space.newmemoryview(self)
 
 
-class RawBufferView_Base(BufferView):
-    """Abstract base class for views into RawBuffers"""
+class SimpleView(BufferView):
     _attrs_ = ['readonly', 'data']
     _immutable_ = True
 
+    def __init__(self, data):
+        self.data = data
+        self.readonly = self.data.readonly
+
     def getlength(self):
         return self.data.getlength()
 
@@ -203,7 +185,7 @@ class RawBufferView_Base(BufferView):
         return self.data[start:start + size]
 
     def setbytes(self, offset, s):
-        return self.data.setslice(offset, s)
+        self.data.setslice(offset, s)
 
     def get_raw_address(self):
         return self.data.get_raw_address()
@@ -215,52 +197,6 @@ class RawBufferView_Base(BufferView):
         assert not self.data.readonly
         return self.data
 
-
-class RawBufferView(RawBufferView_Base):
-    _attrs_ = ['readonly', 'data', 'fmt', 'itemsize']
-    _immutable_ = True
-
-    def __init__(self, data, fmt, itemsize, w_obj=None):
-        assert isinstance(data, RawBuffer)
-        self.data = data
-        self.readonly = data.readonly
-        self.fmt = fmt
-        self.itemsize = itemsize
-        self.w_obj = w_obj
-
-    def getformat(self):
-        return self.fmt
-
-    def getitemsize(self):
-        return self.itemsize
-
-    def getndim(self):
-        return 1
-
-    def getshape(self):
-        return [self.getlength() // self.itemsize]
-
-    def getstrides(self):
-        return [self.getitemsize()]
-
-    def new_slice(self, start, step, slicelength):
-        if step == 1:
-            n = self.itemsize
-            newbuf = SubBuffer(self.data, start * n, slicelength * n)
-            return RawBufferView(newbuf, self.fmt, self.itemsize, w_obj=self.w_obj)
-        else:
-            return BufferView.new_slice(self, start, step, slicelength)
-
-
-class SimpleView(RawBufferView_Base):
-    _attrs_ = ['readonly', 'data']
-    _immutable_ = True
-
-    def __init__(self, data, w_obj=None):
-        self.data = data
-        self.readonly = self.data.readonly
-        self.w_obj = w_obj
-
     def getformat(self):
         return 'B'
 
@@ -290,25 +226,20 @@ class SimpleView(RawBufferView_Base):
     def w_getitem(self, space, idx):
         idx = self.get_offset(space, 0, idx)
         ch = self.data[idx]
-        return space.newint(ord(ch))
+        return space.newbytes(ch)
 
     def new_slice(self, start, step, slicelength):
         if step == 1:
-            return SimpleView(SubBuffer(self.data, start, slicelength), w_obj=self.w_obj)
+            return SimpleView(SubBuffer(self.data, start, slicelength))
         else:
-            return BufferSlice(self, start, step, slicelength, w_obj=self.w_obj)
-
-    def setitem_w(self, space, idx, w_obj):
-        idx = self.get_offset(space, 0, idx)
-        self.data[idx] = space.byte_w(w_obj)
+            return BufferSlice(self, start, step, slicelength)
 
 
 class BufferSlice(BufferView):
     _immutable_ = True
     _attrs_ = ['parent', 'readonly', 'shape', 'strides', 'start', 'step']
 
-    def __init__(self, parent, start, step, length, w_obj=None):
-        self.w_obj = w_obj
+    def __init__(self, parent, start, step, length):
         self.parent = parent
         self.readonly = self.parent.readonly
         self.strides = parent.getstrides()[:]
@@ -326,26 +257,11 @@ class BufferSlice(BufferView):
         return self.parent.getbytes(offset + start, size)
 
     def setbytes(self, start, string):
-        # Unlike getbytes, which always goes through _copy_base,
-        # this is directly exposed. It must keep track of weird strides.
-        length = len(string)
-        if length == 0:
+        if len(string) == 0:
             return        # otherwise, adding self.offset might make 'start'
                           # out of bounds
-        elif length > 1 and self.step != 1:
-            # We cannot use self.parent.setbytes, we need to roll our own
-            # XXX check that length is not too long
-            last_stride = self.getstrides()[0]
-            itemsize = self.getitemsize()
-            assert itemsize >= 0
-            offset = self.start * itemsize
-            for i in range(length):
-                os = offset + i * last_stride
-                start = i * itemsize
-                self.parent.setbytes(os, string[start:(start+itemsize)])
-        else:
-            offset = self.start * self.parent.getstrides()[0]
-            self.parent.setbytes(offset + start, string)
+        offset = self.start * self.parent.getstrides()[0]
+        self.parent.setbytes(offset + start, string)
 
     def get_raw_address(self):
         from rpython.rtyper.lltypesystem import rffi
@@ -377,60 +293,3 @@ class BufferSlice(BufferView):
         real_start = start + self.start
         real_step = self.step * step
         return BufferSlice(self.parent, real_start, real_step, slicelength)
-
-    def setitem_w(self, space, idx, w_obj):
-        return self.parent.setitem_w(space, self.parent_index(idx), w_obj)
-
-
-# XXX not sure this is the right approach, maybe adding a copy to BufferView or
-# even a toreadonly would be a better approach
-
-class ReadonlyWrapper(BufferView):
-    _immutable_ = True
-    def __init__(self, view):
-        self.view = view
-        self.readonly = True
-        self.w_obj = view.w_obj
-
-    def getlength(self):
-        return self.view.getlength()
-
-    def as_str(self):
-        return self.view.as_str()
-
-    def getbytes(self, start, size):
-        return self.view.getbytes(start, size)
-
-    def setbytes(self, start, string):
-        assert 0, "should be unreachable"
-
-    def get_raw_address(self):
-        return self.view.get_raw_address()
-
-    def as_readbuf(self):
-        return self.view.as_readbuf()
-
-    def as_writebuf(self):
-        return self.view.as_writebuf()
-
-    def getformat(self):
-        return self.view.getformat()
-
-    def getitemsize(self):
-        return self.view.getitemsize()
-
-    def getndim(self):
-        return self.view.getndim()
-
-    def getshape(self):
-        return self.view.getshape()
-
-    def getstrides(self):
-        return self.view.getstrides()
-
-    def releasebuffer(self):
-        return self.view.releasebuffer()
-
-    def new_slice(self, start, step, slicelength):
-        return ReadonlyWrapper(BufferSlice(self, start, step, slicelength, w_obj=self.w_obj))
-
