@@ -1,5 +1,5 @@
-from pypy.interpreter.error import OperationError, oefmt, wrap_oserror
-from pypy.interpreter.gateway import WrappedDefault, unwrap_spec
+from pypy.interpreter.error import oefmt, wrap_oserror
+from pypy.interpreter.gateway import unwrap_spec
 from pypy.interpreter.pycode import CodeHookCache
 from pypy.interpreter.pyframe import PyFrame
 from pypy.interpreter.mixedmodule import MixedModule
@@ -10,20 +10,11 @@ from pypy.objspace.std.setobject import W_BaseSetObject
 from pypy.objspace.std.typeobject import MethodCache
 from pypy.objspace.std.mapdict import MapAttrCache
 from rpython.rlib import rposix, rgc, rstack
-from rpython.rtyper.lltypesystem import rffi
 
 
 def internal_repr(space, w_object):
     return space.newtext('%r' % (w_object,))
 
-def objects_in_repr(space):
-    """The identitydict of objects currently being repr().
-
-    This object is thread-local and can be used in a __repr__ method
-    to avoid recursion.
-    """
-    return space.get_objects_in_repr()
-
 
 def attach_gdb(space):
     """Run an interp-level gdb (or pdb when untranslated)"""
@@ -37,8 +28,8 @@ def method_cache_counter(space, name):
     methods with the name."""
     assert space.config.objspace.std.withmethodcachecounter
     cache = space.fromcache(MethodCache)
-    return space.newtuple2(space.newint(cache.hits.get(name, 0)),
-                           space.newint(cache.misses.get(name, 0)))
+    return space.newtuple([space.newint(cache.hits.get(name, 0)),
+                           space.newint(cache.misses.get(name, 0))])
 
 def reset_method_cache_counter(space):
     """Reset the method cache counter to zero for all method names."""
@@ -56,8 +47,8 @@ def mapdict_cache_counter(space, name):
     in the mapdict cache with the given attribute name."""
     assert space.config.objspace.std.withmethodcachecounter
     cache = space.fromcache(MapAttrCache)
-    return space.newtuple2(space.newint(cache.hits.get(name, 0)),
-                           space.newint(cache.misses.get(name, 0)))
+    return space.newtuple([space.newint(cache.hits.get(name, 0)),
+                           space.newint(cache.misses.get(name, 0))])
 
 def builtinify(space, w_func):
     """To implement at app-level modules that are, in CPython,
@@ -77,25 +68,32 @@ def hidden_applevel(space, w_func):
     func.getcode().hidden_applevel = True
     return w_func
 
+def get_hidden_tb(space):
+    """Return the traceback of the current exception being handled by a
+    frame hidden from applevel.
+    """
+    operr = space.getexecutioncontext().sys_exc_info(for_hidden=True)
+    return space.w_None if operr is None else operr.get_w_traceback(space)
+
 @unwrap_spec(meth='text')
 def lookup_special(space, w_obj, meth):
     """Lookup up a special method on an object."""
+    if space.is_oldstyle_instance(w_obj):
+        raise oefmt(space.w_TypeError,
+                    "this doesn't do what you want on old-style classes")
     w_descr = space.lookup(w_obj, meth)
     if w_descr is None:
         return space.w_None
     return space.get(w_descr, w_obj)
 
-def do_what_I_mean(space):
-    "Return 42"
+def do_what_I_mean(space, w_crash=None):
+    if not space.is_none(w_crash):
+        raise ValueError    # RPython-level, uncaught
     return space.newint(42)
 
-def _internal_crash(space, w_crash=None):
-    """for testing purposes, raise an interpreter-level ValueError. Should turn
-    into a SystemError automatically"""
-    raise ValueError    # RPython-level, uncaught
 
 def strategy(space, w_obj):
-    """ strategy(dict or list or set or instance)
+    """ strategy(dict or list or set)
 
     Return the underlying strategy currently used by a dict, list or set object
     """
@@ -106,43 +104,24 @@ def strategy(space, w_obj):
     elif isinstance(w_obj, W_BaseSetObject):
         name = w_obj.strategy.__class__.__name__
     else:
-        m = w_obj._get_mapdict_map()
-        if m is not None:
-            name = m.repr()
-        else:
-            raise oefmt(space.w_TypeError, "expecting dict or list or set object, or instance of some kind")
+        raise oefmt(space.w_TypeError, "expecting dict or list or set object")
     return space.newtext(name)
 
-def list_get_physical_size(space, w_obj):
-    if not isinstance(w_obj, W_ListObject):
-        raise oefmt(space.w_TypeError, "expected list")
-    return space.newint(w_obj.physical_size())
 
-
-def get_console_cp(space):
-    """get_console_cp()
-
-    Return the console and console output code page (windows only)
-    """
-    from rpython.rlib import rwin32    # Windows only
-    return space.newtuple2(
-        space.newtext('cp%d' % rwin32.GetConsoleCP()),
-        space.newtext('cp%d' % rwin32.GetConsoleOutputCP()),
-        )
-
-@unwrap_spec(fd=int)
-def get_osfhandle(space, fd):
-    """get_osfhandle()
-
-    Return the handle corresponding to the file descriptor (windows only)
-    """
-    from rpython.rlib import rwin32    # Windows only
+@unwrap_spec(fd='c_int')
+def validate_fd(space, fd):
     try:
-        ret = rwin32.get_osfhandle(fd)
-        return space.newint(rffi.cast(rffi.INT, ret))
+        rposix.validate_fd(fd)
     except OSError as e:
         raise wrap_oserror(space, e)
 
+def get_console_cp(space):
+    from rpython.rlib import rwin32    # Windows only
+    return space.newtuple([
+        space.newtext('cp%d' % rwin32.GetConsoleCP()),
+        space.newtext('cp%d' % rwin32.GetConsoleOutputCP()),
+        ])
+
 @unwrap_spec(sizehint=int)
 def resizelist_hint(space, w_list, sizehint):
     """ Reallocate the underlying storage of the argument list to sizehint """
@@ -155,8 +134,15 @@ def newlist_hint(space, sizehint):
     """ Create a new empty list that has an underlying storage of length sizehint """
     return space.newlist_hint(sizehint)
 
+@unwrap_spec(debug=bool)
+def set_debug(space, debug):
+    space.sys.debug = debug
+    space.setitem(space.builtin.w_dict,
+                  space.newtext('__debug__'),
+                  space.newbool(debug))
+
 @unwrap_spec(estimate=int)
-def add_memory_pressure(space, estimate):
+def add_memory_pressure(estimate):
     """ Add memory pressure of estimate bytes. Useful when calling a C function
     that internally allocates a big chunk of memory. This instructs the GC to
     garbage collect sooner than it would otherwise."""
@@ -167,6 +153,14 @@ def locals_to_fast(space, w_frame):
     assert isinstance(w_frame, PyFrame)
     w_frame.locals2fast()
 
+@unwrap_spec(w_module=MixedModule)
+def save_module_content_for_future_reload(space, w_module):
+    w_module.save_module_content_for_future_reload()
+
+def specialized_zip_2_lists(space, w_list1, w_list2):
+    from pypy.objspace.std.specialisedtupleobject import specialized_zip_2_lists
+    return specialized_zip_2_lists(space, w_list1, w_list2)
+
 def set_code_callback(space, w_callable):
     cache = space.fromcache(CodeHookCache)
     if space.is_none(w_callable):
@@ -202,112 +196,6 @@ def _promote(space, w_obj):
         jit.promote(w_obj)
     return w_obj
 
-@unwrap_spec(w_value=WrappedDefault(None), w_tb=WrappedDefault(None))
-def normalize_exc(space, w_type, w_value=None, w_tb=None):
-    operr = OperationError(w_type, w_value, w_tb)
-    operr.normalize_exception(space)
-    return operr.get_w_value(space)
-
 def stack_almost_full(space):
     """Return True if the stack is more than 15/16th full."""
     return space.newbool(rstack.stack_almost_full())
-
-def fsencode(space, w_obj):
-    """Direct access to the interp-level fsencode()"""
-    return space.fsencode(w_obj)
-
-def fsdecode(space, w_obj):
-    """Direct access to the interp-level fsdecode()"""
-    return space.fsdecode(w_obj)
-
-def side_effects_ok(space):
-    """For use with the reverse-debugger: this function normally returns
-    True, but will return False if we are evaluating a debugging command
-    like a watchpoint.  You are responsible for not doing any side effect
-    at all (including no caching) when evaluating watchpoints.  This
-    function is meant to help a bit---you can write:
-
-        if not __pypy__.side_effects_ok():
-            skip the caching logic
-
-    inside getter methods or properties, to make them usable from
-    watchpoints.  Note that you need to re-run ``REVDB=.. pypy''
-    after changing the Python code.
-    """
-    return space.newbool(space._side_effects_ok())
-
-def revdb_stop(space):
-    from pypy.interpreter.reverse_debugging import stop_point
-    stop_point()
-
-def pyos_inputhook(space):
-    """Call PyOS_InputHook() from the CPython C API."""
-    if not space.config.objspace.usemodules.cpyext:
-        return
-    w_modules = space.sys.get('modules')
-    if space.finditem_str(w_modules, 'cpyext') is None:
-        return      # cpyext not imported yet, ignore
-    from pypy.module.cpyext.api import invoke_pyos_inputhook
-    invoke_pyos_inputhook(space)
-
-def utf8content(space, w_u):
-    """ Given a unicode string u, return it's internal byte representation.
-    Useful for debugging only. """
-    from pypy.objspace.std.unicodeobject import W_UnicodeObject
-    if type(w_u) is not W_UnicodeObject:
-        raise oefmt(space.w_TypeError, "expected unicode string, got %T", w_u)
-    return space.newbytes(w_u._utf8)
-
-def set_exc_info(space, w_type, w_value, w_traceback=None):
-    ec = space.getexecutioncontext()
-    ec.set_sys_exc_info3(w_type, w_value, w_traceback)
-
-def get_contextvar_context(space):
-    ec = space.getexecutioncontext()
-    context = ec.contextvar_context
-    if context:
-        return context
-    else:
-        return space.w_None
-
-def set_contextvar_context(space, w_obj):
-    ec = space.getexecutioncontext()
-    ec.contextvar_context = w_obj
-    return space.w_None
-
-def set_exc_info(space, w_type, w_value, w_traceback=None):
-    ec = space.getexecutioncontext()
-    ec.set_sys_exc_info3(w_type, w_value, w_traceback)
-
-def get_contextvar_context(space):
-    ec = space.getexecutioncontext()
-    context = ec.contextvar_context
-    if context:
-        return context
-    else:
-        return space.w_None
-
-def set_contextvar_context(space, w_obj):
-    ec = space.getexecutioncontext()
-    ec.contextvar_context = w_obj
-    return space.w_None
-
-
-@unwrap_spec(where='text')
-def write_unraisable(space, where, w_exc, w_obj):
-    """write_unraisable(where, exc, obj)
-       Equivalent to CPython's _PyErr_WriteUnraisableMsg()
-
-       where: msg to write (text)
-       exc:   error raised
-       obj:   object to print its repr
-    """
-    OperationError(space.type(w_exc), w_exc).write_unraisable(
-                            space, where, w_obj, with_traceback=True)
-
-def _testing_clear_audithooks(space):
-    if we_are_translated():
-        raise oefmt(space.w_RuntimeError, "can only use _testing_clear_audithooks before translation")
-    from pypy.module.sys.vm import AuditHolder
-    holder = space.fromcache(AuditHolder)
-    holder.hook_chain = None
