@@ -1,5 +1,5 @@
 from rpython.rlib import jit, rgc
-from rpython.rlib.buffer import Buffer
+from rpython.rlib.buffer import RawBuffer
 from rpython.rlib.objectmodel import keepalive_until_here
 from rpython.rlib.rarithmetic import ovfcheck, widen
 from rpython.rlib.unroll import unrolling_iterable
@@ -16,7 +16,7 @@ from pypy.interpreter.typedef import (
 from pypy.module._file.interp_file import W_File
 
 
-@unwrap_spec(typecode=str)
+@unwrap_spec(typecode='text')
 def w_array(space, w_cls, typecode, __args__):
     if len(__args__.arguments_w) > 1:
         raise oefmt(space.w_TypeError, "array() takes at most 2 arguments")
@@ -43,7 +43,7 @@ def w_array(space, w_cls, typecode, __args__):
     if len(__args__.arguments_w) > 0:
         w_initializer = __args__.arguments_w[0]
         w_initializer_type = space.type(w_initializer)
-        if w_initializer_type is space.w_str:
+        if w_initializer_type is space.w_bytes:
             a.descr_fromstring(space, w_initializer)
         elif w_initializer_type is space.w_list:
             a.descr_fromlist(space, w_initializer)
@@ -53,11 +53,11 @@ def w_array(space, w_cls, typecode, __args__):
 
 
 def descr_itemsize(space, self):
-    return space.wrap(self.itemsize)
+    return space.newint(self.itemsize)
 
 
 def descr_typecode(space, self):
-    return space.wrap(self.typecode)
+    return space.newtext(self.typecode)
 
 arr_eq_driver = jit.JitDriver(name='array_eq_driver', greens=['comp_func'],
                               reds='auto')
@@ -76,7 +76,7 @@ def compare_arrays(space, arr1, arr2, comp_op):
         w_elem1 = arr1.w_getitem(space, i)
         w_elem2 = arr2.w_getitem(space, i)
         if comp_op == EQ:
-            res = space.is_true(space.eq(w_elem1, w_elem2))
+            res = space.eq_w(w_elem1, w_elem2)
             if not res:
                 return space.w_False
         elif comp_op == NE:
@@ -90,7 +90,7 @@ def compare_arrays(space, arr1, arr2, comp_op):
                 res = space.is_true(space.gt(w_elem1, w_elem2))
             if res:
                 return space.w_True
-            elif not space.is_true(space.eq(w_elem1, w_elem2)):
+            elif not space.eq_w(w_elem1, w_elem2):
                 return space.w_False
         else:
             if comp_op == LE:
@@ -99,7 +99,7 @@ def compare_arrays(space, arr1, arr2, comp_op):
                 res = space.is_true(space.ge(w_elem1, w_elem2))
             if not res:
                 return space.w_False
-            elif not space.is_true(space.eq(w_elem1, w_elem2)):
+            elif not space.eq_w(w_elem1, w_elem2):
                 return space.w_True
     # we have some leftovers
     if comp_op == EQ:
@@ -118,16 +118,147 @@ def compare_arrays(space, arr1, arr2, comp_op):
         return space.w_True
     return space.w_False
 
+index_count_jd = jit.JitDriver(
+    greens = ['count', 'arrclass', 'tp_item'],
+    reds = 'auto', name = 'array.index_or_count')
+
+def index_count_array(arr, w_val, count=False):
+    space = arr.space
+    tp_item = space.type(w_val)
+    arrclass = arr.__class__
+    cnt = 0
+    for i in range(arr.len):
+        index_count_jd.jit_merge_point(
+            tp_item=tp_item, count=count,
+            arrclass=arrclass)
+        w_item = arr.w_getitem(space, i)
+        if space.eq_w(w_item, w_val):
+            if count:
+                cnt += 1
+            else:
+                return i
+    if count:
+        return cnt
+    return -1
+
 UNICODE_ARRAY = lltype.Ptr(lltype.Array(lltype.UniChar,
                                         hints={'nolength': True}))
 
 class W_ArrayBase(W_Root):
-    _attrs_ = ('space', 'len', 'allocated', '_lifeline_') # no buffer
+    _attrs_ = ('space', 'len', 'allocated', '_lifeline_', '_buffer')
 
     def __init__(self, space):
         self.space = space
         self.len = 0
         self.allocated = 0
+        self._buffer = lltype.nullptr(rffi.CCHARP.TO)
+
+    @rgc.must_be_light_finalizer
+    def __del__(self):
+        if self._buffer:
+            lltype.free(self._buffer, flavor='raw')
+
+    def setlen(self, size, zero=False, overallocate=True):
+        if size > 0:
+            if size > self.allocated or size < self.allocated / 2:
+                if overallocate:
+                    if size < 9:
+                        some = 3
+                    else:
+                        some = 6
+                    some += size >> 3
+                else:
+                    some = 0
+                self.allocated = size + some
+                byte_size = self.allocated * self.itemsize
+                if zero:
+                    new_buffer = lltype.malloc(
+                        rffi.CCHARP.TO, byte_size, flavor='raw',
+                        add_memory_pressure=True, zero=True)
+                else:
+                    new_buffer = lltype.malloc(
+                        rffi.CCHARP.TO, byte_size, flavor='raw',
+                        add_memory_pressure=True)
+                    copy_bytes = min(size, self.len) * self.itemsize
+                    rffi.c_memcpy(rffi.cast(rffi.VOIDP, new_buffer),
+                                  rffi.cast(rffi.VOIDP, self._buffer),
+                                  copy_bytes)
+            else:
+                self.len = size
+                return
+        else:
+            assert size == 0
+            self.allocated = 0
+            new_buffer = lltype.nullptr(rffi.CCHARP.TO)
+
+        if self._buffer:
+            lltype.free(self._buffer, flavor='raw')
+        self._buffer = new_buffer
+        self.len = size
+
+    def _fromiterable(self, w_seq):
+        # used by fromsequence().
+        # a more careful case if w_seq happens to be a very large
+        # iterable: don't copy the items into some intermediate list
+        w_iterator = self.space.iter(w_seq)
+        tp = self.space.type(w_iterator)
+        while True:
+            unpack_driver.jit_merge_point(selfclass=self.__class__,
+                                          tp=tp, self=self,
+                                          w_iterator=w_iterator)
+            space = self.space
+            try:
+                w_item = space.next(w_iterator)
+            except OperationError as e:
+                if not e.match(space, space.w_StopIteration):
+                    raise
+                break  # done
+            self.descr_append(space, w_item)
+
+    def _charbuf_start(self):
+        return self._buffer
+
+    def _buffer_as_unsigned(self):
+        return rffi.cast(lltype.Unsigned, self._buffer)
+
+    def _charbuf_stop(self):
+        keepalive_until_here(self)
+
+    def delitem(self, space, i, j):
+        if i < 0:
+            i += self.len
+        if i < 0:
+            i = 0
+        if j < 0:
+            j += self.len
+        if j < 0:
+            j = 0
+        if j > self.len:
+            j = self.len
+        if i >= j:
+            return None
+        oldbuffer = self._buffer
+        self._buffer = lltype.malloc(rffi.CCHARP.TO,
+            (self.len - (j - i)) * self.itemsize, flavor='raw',
+            add_memory_pressure=True)
+        if i:
+            rffi.c_memcpy(
+                rffi.cast(rffi.VOIDP, self._buffer),
+                rffi.cast(rffi.VOIDP, oldbuffer),
+                i * self.itemsize
+            )
+        if j < self.len:
+            rffi.c_memcpy(
+                rffi.cast(rffi.VOIDP, rffi.ptradd(self._buffer,
+                                                  i * self.itemsize)),
+                rffi.cast(rffi.VOIDP, rffi.ptradd(oldbuffer,
+                                                  j * self.itemsize)),
+                (self.len - j) * self.itemsize
+            )
+        self.len -= j - i
+        self.allocated = self.len
+        if oldbuffer:
+            lltype.free(oldbuffer, flavor='raw')
 
     def readbuf_w(self, space):
         return ArrayBuffer(self, True)
@@ -149,19 +280,23 @@ class W_ArrayBase(W_Root):
         """
         self.extend(w_x)
 
-    def descr_count(self, space, w_val):
+    def descr_count(self, space, w_x):
         """ count(x)
 
         Return number of occurrences of x in the array.
         """
-        raise NotImplementedError
+        cnt = index_count_array(self, w_x, count=True)
+        return space.newint(cnt)
 
     def descr_index(self, space, w_x):
         """ index(x)
 
         Return index of first occurrence of x in the array.
         """
-        raise NotImplementedError
+        res = index_count_array(self, w_x, count=False)
+        if res >= 0:
+            return space.newint(res)
+        raise oefmt(space.w_ValueError, "array.index(x): x not in list")
 
     def descr_reverse(self, space):
         """ reverse()
@@ -175,7 +310,8 @@ class W_ArrayBase(W_Root):
 
         Remove the first occurrence of x in the array.
         """
-        raise NotImplementedError
+        w_idx = self.descr_index(space, w_val)
+        self.descr_pop(space, space.int_w(w_idx))
 
     @unwrap_spec(i=int)
     def descr_pop(self, space, i=-1):
@@ -265,14 +401,14 @@ class W_ArrayBase(W_Root):
             size = ovfcheck(self.itemsize * n)
         except OverflowError:
             raise MemoryError
-        w_item = space.call_method(w_f, 'read', space.wrap(size))
+        w_item = space.call_method(w_f, 'read', space.newint(size))
         item = space.bytes_w(w_item)
         if len(item) < size:
             n = len(item) % self.itemsize
             elems = max(0, len(item) - (len(item) % self.itemsize))
             if n != 0:
                 item = item[0:elems]
-            self.descr_fromstring(space, space.wrap(item))
+            self.descr_fromstring(space, space.newbytes(item))
             raise oefmt(space.w_EOFError, "not enough items in file")
         self.descr_fromstring(space, w_item)
 
@@ -315,7 +451,7 @@ class W_ArrayBase(W_Root):
         """
         if self.typecode == 'u':
             buf = rffi.cast(UNICODE_ARRAY, self._buffer_as_unsigned())
-            return space.wrap(rffi.wcharpsize2unicode(buf, self.len))
+            return space.newunicode(rffi.wcharpsize2unicode(buf, self.len))
         else:
             raise oefmt(space.w_ValueError,
                         "tounicode() may only be called on type 'u' arrays")
@@ -328,8 +464,8 @@ class W_ArrayBase(W_Root):
         The length should be multiplied by the itemsize attribute to calculate
         the buffer length in bytes.
         """
-        w_ptr = space.wrap(self._buffer_as_unsigned())
-        w_len = space.wrap(self.len)
+        w_ptr = space.newint(self._buffer_as_unsigned())
+        w_len = space.newint(self.len)
         return space.newtuple([w_ptr, w_len])
 
     def descr_reduce(self, space):
@@ -337,11 +473,11 @@ class W_ArrayBase(W_Root):
         """
         if self.len > 0:
             w_s = self.descr_tostring(space)
-            args = [space.wrap(self.typecode), w_s]
+            args = [space.newtext(self.typecode), w_s]
         else:
-            args = [space.wrap(self.typecode)]
+            args = [space.newtext(self.typecode)]
         try:
-            w_dict = space.getattr(self, space.wrap('__dict__'))
+            w_dict = space.getattr(self, space.newtext('__dict__'))
         except OperationError:
             w_dict = space.w_None
         return space.newtuple([space.type(self), space.newtuple(args), w_dict])
@@ -382,7 +518,7 @@ class W_ArrayBase(W_Root):
         self._charbuf_stop()
 
     def descr_len(self, space):
-        return space.wrap(self.len)
+        return space.newint(self.len)
 
     def descr_eq(self, space, w_arr2):
         "x.__eq__(y) <==> x==y"
@@ -453,16 +589,102 @@ class W_ArrayBase(W_Root):
         return space.newseqiter(self)
 
     def descr_add(self, space, w_other):
-        raise NotImplementedError
+        if (not isinstance(w_other, W_ArrayBase)
+                or w_other.typecode != self.typecode):
+            return space.w_NotImplemented
+        a = self.constructor(space)
+        a.setlen(self.len + w_other.len, overallocate=False)
+        if self.len:
+            rffi.c_memcpy(
+                rffi.cast(rffi.VOIDP, a._buffer),
+                rffi.cast(rffi.VOIDP, self._buffer),
+                self.len * self.itemsize
+            )
+        if w_other.len:
+            rffi.c_memcpy(
+                rffi.cast(rffi.VOIDP, rffi.ptradd(a._buffer,
+                                             self.len * self.itemsize)),
+                rffi.cast(rffi.VOIDP, w_other._buffer),
+                w_other.len * self.itemsize
+            )
+        keepalive_until_here(self)
+        keepalive_until_here(a)
+        return a
 
     def descr_inplace_add(self, space, w_other):
-        raise NotImplementedError
+        if (not isinstance(w_other, W_ArrayBase)
+                or w_other.typecode != self.typecode):
+            return space.w_NotImplemented
+        oldlen = self.len
+        otherlen = w_other.len
+        self.setlen(oldlen + otherlen)
+        if otherlen:
+            rffi.c_memcpy(
+                rffi.cast(rffi.VOIDP, rffi.ptradd(self._buffer,
+                                             oldlen * self.itemsize)),
+                rffi.cast(rffi.VOIDP, w_other._buffer),
+                otherlen * self.itemsize
+            )
+        keepalive_until_here(self)
+        keepalive_until_here(w_other)
+        return self
+
+    def _mul_helper(self, space, w_repeat, is_inplace):
+        try:
+            repeat = space.getindex_w(w_repeat, space.w_OverflowError)
+        except OperationError as e:
+            if e.match(space, space.w_TypeError):
+                return space.w_NotImplemented
+            raise
+        if is_inplace:
+            a = self
+            start = 1
+        else:
+            a = self.constructor(space)
+            start = 0
+        if repeat <= start:
+            if repeat <= 0:
+                a.setlen(0, overallocate=False)
+            return a
+        oldlen = self.len
+        try:
+            newlen = ovfcheck(oldlen * repeat)
+        except OverflowError:
+            raise MemoryError
+        #
+        srcbuf = self._buffer
+        srcsize = self.len * self.itemsize
+        for i in range(srcsize):
+            if srcbuf[i] != '\x00':
+                break
+        else:
+            # the source is entirely zero: initialize the target
+            # with zeroes too
+            a.setlen(newlen, zero=True, overallocate=False)
+            return a
+        #
+        a.setlen(newlen, overallocate=False)
+        srcbuf = self._buffer   # reload this, in case self is a
+        if oldlen == 1:
+            self._repeat_single_item(a, start, repeat)
+        else:
+            dstbuf = a._buffer
+            if start == 1:
+                dstbuf = rffi.ptradd(dstbuf, srcsize)
+            for r in range(start, repeat):
+                rffi.c_memcpy(rffi.cast(rffi.VOIDP, dstbuf),
+                              rffi.cast(rffi.VOIDP, srcbuf),
+                              srcsize)
+                dstbuf = rffi.ptradd(dstbuf, srcsize)
+        keepalive_until_here(self)
+        keepalive_until_here(a)
+        return a
 
     def descr_mul(self, space, w_repeat):
-        raise NotImplementedError
+        return self._mul_helper(space, w_repeat, False)
 
     def descr_inplace_mul(self, space, w_repeat):
-        raise NotImplementedError
+        return self._mul_helper(space, w_repeat, True)
 
     def descr_radd(self, space, w_other):
         return self.descr_add(space, w_other)
@@ -474,19 +696,19 @@ class W_ArrayBase(W_Root):
 
     def descr_repr(self, space):
         if self.len == 0:
-            return space.wrap("array('%s')" % self.typecode)
+            return space.newtext("array('%s')" % self.typecode)
         elif self.typecode == "c":
             r = space.repr(self.descr_tostring(space))
-            s = "array('%s', %s)" % (self.typecode, space.str_w(r))
-            return space.wrap(s)
+            s = "array('%s', %s)" % (self.typecode, space.text_w(r))
+            return space.newtext(s)
         elif self.typecode == "u":
             r = space.repr(self.descr_tounicode(space))
-            s = "array('%s', %s)" % (self.typecode, space.str_w(r))
-            return space.wrap(s)
+            s = "array('%s', %s)" % (self.typecode, space.text_w(r))
+            return space.newtext(s)
         else:
             r = space.repr(self.descr_tolist(space))
-            s = "array('%s', %s)" % (self.typecode, space.str_w(r))
-            return space.wrap(s)
+            s = "array('%s', %s)" % (self.typecode, space.text_w(r))
+            return space.newtext(s)
 
 W_ArrayBase.typedef = TypeDef(
     'array.array',
@@ -547,15 +769,19 @@ W_ArrayBase.typedef = TypeDef(
 
 class TypeCode(object):
     def __init__(self, itemtype, unwrap, canoverflow=False, signed=False,
-                 method='__int__'):
+                 method='__int__', errorname=None):
+        if errorname is None:
+            errorname = unwrap[:-2]
         self.itemtype = itemtype
         self.bytes = rffi.sizeof(itemtype)
         self.arraytype = lltype.Array(itemtype, hints={'nolength': True})
+        self.arrayptrtype = lltype.Ptr(self.arraytype)
         self.unwrap = unwrap
         self.signed = signed
         self.canoverflow = canoverflow
         self.w_class = None
         self.method = method
+        self.errorname = errorname
 
     def _freeze_(self):
         # hint for the annotator: track individual constant instances
@@ -570,7 +796,7 @@ else:
     _UINTTypeCode = \
          TypeCode(rffi.UINT,          'int_w', True)
 types = {
-    'c': TypeCode(lltype.Char,        'str_w', method=''),
+    'c': TypeCode(lltype.Char,        'bytes_w', method=''),
     'u': TypeCode(lltype.UniChar,     'unicode_w', method=''),
     'b': TypeCode(rffi.SIGNEDCHAR,    'int_w', True, True),
     'B': TypeCode(rffi.UCHAR,         'int_w', True),
@@ -579,8 +805,8 @@ types = {
     'i': TypeCode(rffi.INT,           'int_w', True, True),
     'I': _UINTTypeCode,
     'l': TypeCode(rffi.LONG,          'int_w', True, True),
-    'L': TypeCode(rffi.ULONG,         'bigint_w'),  # Overflow handled by
-                                                    # rbigint.touint() which
+    'L': TypeCode(rffi.ULONG,         'bigint_w',   # Overflow handled by
+                  errorname="integer"),             # rbigint.touint() which
                                                     # corresponds to the
                                                     # C-type unsigned long
     'f': TypeCode(lltype.SingleFloat, 'float_w', method='__float__'),
@@ -590,72 +816,59 @@ for k, v in types.items():
     v.typecode = k
 unroll_typecodes = unrolling_iterable(types.keys())
 
-class ArrayBuffer(Buffer):
+class ArrayBuffer(RawBuffer):
     _immutable_ = True
 
-    def __init__(self, array, readonly):
-        self.array = array
+    def __init__(self, w_array, readonly):
+        self.w_array = w_array
         self.readonly = readonly
 
     def getlength(self):
-        return self.array.len * self.array.itemsize
-
-    def getformat(self):
-        return self.array.typecode
-
-    def getitemsize(self):
-        return self.array.itemsize
-
-    def getndim(self):
-        return 1
-
-    def getstrides(self):
-        return [self.getitemsize()]
+        return self.w_array.len * self.w_array.itemsize
 
     def getitem(self, index):
-        array = self.array
-        data = array._charbuf_start()
+        w_array = self.w_array
+        data = w_array._charbuf_start()
         char = data[index]
-        array._charbuf_stop()
+        w_array._charbuf_stop()
         return char
 
     def setitem(self, index, char):
-        array = self.array
-        data = array._charbuf_start()
+        w_array = self.w_array
+        data = w_array._charbuf_start()
         data[index] = char
-        array._charbuf_stop()
+        w_array._charbuf_stop()
 
     def getslice(self, start, stop, step, size):
         if size == 0:
             return ''
         if step == 1:
-            data = self.array._charbuf_start()
+            data = self.w_array._charbuf_start()
             try:
                 return rffi.charpsize2str(rffi.ptradd(data, start), size)
             finally:
-                self.array._charbuf_stop()
-        return Buffer.getslice(self, start, stop, step, size)
+                self.w_array._charbuf_stop()
+        return RawBuffer.getslice(self, start, stop, step, size)
 
     def get_raw_address(self):
-        return self.array._charbuf_start()
+        return self.w_array._charbuf_start()
 
 
+unpack_driver = jit.JitDriver(name='unpack_array',
+                              greens=['selfclass', 'tp'],
+                              reds=['self', 'w_iterator'])
+
 def make_array(mytype):
     W_ArrayBase = globals()['W_ArrayBase']
 
-    unpack_driver = jit.JitDriver(name='unpack_array',
-                                  greens=['tp'],
-                                  reds=['self', 'w_iterator'])
-
     class W_Array(W_ArrayBase):
         itemsize = mytype.bytes
         typecode = mytype.typecode
 
-        _attrs_ = ('space', 'len', 'allocated', '_lifeline_', 'buffer')
+        _attrs_ = W_ArrayBase._attrs_
 
-        def __init__(self, space):
-            W_ArrayBase.__init__(self, space)
-            self.buffer = lltype.nullptr(mytype.arraytype)
+        def get_buffer(self):
+            return rffi.cast(mytype.arrayptrtype, self._buffer)
 
         def item_w(self, w_item):
             space = self.space
@@ -671,7 +884,7 @@ def make_array(mytype):
                         item = unwrap(space.call_method(w_item, mytype.method))
                     except OperationError:
                         raise oefmt(space.w_TypeError,
-                                    "array item must be " + mytype.unwrap[:-2])
+                                    "array item must be " + mytype.errorname)
                 else:
                     raise
             if mytype.unwrap == 'bigint_w':
@@ -682,7 +895,7 @@ def make_array(mytype):
                                 "unsigned %d-byte integer out of range",
                                 mytype.bytes)
                 return rffi.cast(mytype.itemtype, item)
-            if mytype.unwrap == 'str_w' or mytype.unwrap == 'unicode_w':
+            if mytype.unwrap == 'bytes_w' or mytype.unwrap == 'unicode_w':
                 if len(item) != 1:
                     raise oefmt(space.w_TypeError, "array item must be char")
                 item = item[0]
@@ -706,49 +919,9 @@ def make_array(mytype):
                     if not mytype.signed:
                         msg = 'un' + msg      # 'signed' => 'unsigned'
                     raise OperationError(self.space.w_OverflowError,
-                                         self.space.wrap(msg))
+                                         self.space.newtext(msg))
             return result
 
-        @rgc.must_be_light_finalizer
-        def __del__(self):
-            if self.buffer:
-                lltype.free(self.buffer, flavor='raw')
-
-        def setlen(self, size, zero=False, overallocate=True):
-            if size > 0:
-                if size > self.allocated or size < self.allocated / 2:
-                    if overallocate:
-                        if size < 9:
-                            some = 3
-                        else:
-                            some = 6
-                        some += size >> 3
-                    else:
-                        some = 0
-                    self.allocated = size + some
-                    if zero:
-                        new_buffer = lltype.malloc(
-                            mytype.arraytype, self.allocated, flavor='raw',
-                            add_memory_pressure=True, zero=True)
-                    else:
-                        new_buffer = lltype.malloc(
-                            mytype.arraytype, self.allocated, flavor='raw',
-                            add_memory_pressure=True)
-                        for i in range(min(size, self.len)):
-                            new_buffer[i] = self.buffer[i]
-                else:
-                    self.len = size
-                    return
-            else:
-                assert size == 0
-                self.allocated = 0
-                new_buffer = lltype.nullptr(mytype.arraytype)
-
-            if self.buffer:
-                lltype.free(self.buffer, flavor='raw')
-            self.buffer = new_buffer
-            self.len = size
-
         def fromsequence(self, w_seq):
             space = self.space
             oldlen = self.len
@@ -764,20 +937,21 @@ def make_array(mytype):
             if lst is not None:
                 self.setlen(oldlen + len(lst))
                 try:
-                    buf = self.buffer
+                    buf = self.get_buffer()
                     for num in lst:
                         buf[newlen] = self.item_from_int_or_float(num)
                         newlen += 1
                 except OperationError:
                     self.setlen(newlen)
                     raise
+                keepalive_until_here(self)
                 return
 
             # this is the common case: w_seq is a list or a tuple
             lst_w = space.listview_no_unpack(w_seq)
             if lst_w is not None:
                 self.setlen(oldlen + len(lst_w))
-                buf = self.buffer
+                buf = self.get_buffer()
                 try:
                     for w_num in lst_w:
                         # note: self.item_w() might invoke arbitrary code.
@@ -788,30 +962,14 @@ def make_array(mytype):
                         buf[newlen] = self.item_w(w_num)
                         newlen += 1
                 except OperationError:
-                    if buf == self.buffer:
+                    if buf == self.get_buffer():
                         self.setlen(newlen)
                     raise
+                keepalive_until_here(self)
                 return
 
             self._fromiterable(w_seq)
 
-        def _fromiterable(self, w_seq):
-            # a more careful case if w_seq happens to be a very large
-            # iterable: don't copy the items into some intermediate list
-            w_iterator = self.space.iter(w_seq)
-            tp = self.space.type(w_iterator)
-            while True:
-                unpack_driver.jit_merge_point(tp=tp, self=self,
-                                              w_iterator=w_iterator)
-                space = self.space
-                try:
-                    w_item = space.next(w_iterator)
-                except OperationError as e:
-                    if not e.match(space, space.w_StopIteration):
-                        raise
-                    break  # done
-                self.descr_append(space, w_item)
-
         def extend(self, w_iterable, accept_different_array=False):
             space = self.space
             if isinstance(w_iterable, W_Array):
@@ -819,11 +977,14 @@ def make_array(mytype):
                 new = w_iterable.len
                 self.setlen(self.len + new)
                 i = 0
+                buf = self.get_buffer()
+                srcbuf = w_iterable.get_buffer()
                 while i < new:
                     if oldlen + i >= self.len:
                         self.setlen(oldlen + i + 1)
-                    self.buffer[oldlen + i] = w_iterable.buffer[i]
+                    buf[oldlen + i] = srcbuf[i]
                     i += 1
+                keepalive_until_here(w_iterable)
                 self.setlen(oldlen + i)
             elif (not accept_different_array
                   and isinstance(w_iterable, W_ArrayBase)):
@@ -832,22 +993,22 @@ def make_array(mytype):
             else:
                 self.fromsequence(w_iterable)
 
-        def _charbuf_start(self):
-            return rffi.cast(rffi.CCHARP, self.buffer)
-
-        def _buffer_as_unsigned(self):
-            return rffi.cast(lltype.Unsigned, self.buffer)
-
-        def _charbuf_stop(self):
-            keepalive_until_here(self)
-
         def w_getitem(self, space, idx):
-            item = self.buffer[idx]
+            item = self.get_buffer()[idx]
+            keepalive_until_here(self)
             if mytype.typecode in 'bBhHil':
                 item = rffi.cast(lltype.Signed, item)
-            elif mytype.typecode == 'f':
+                return space.newint(item)
+            if mytype.typecode in 'IL':
+                return space.newint(item)
+            elif mytype.typecode in 'fd':
                 item = float(item)
-            return space.wrap(item)
+                return space.newfloat(item)
+            elif mytype.typecode == 'c':
+                return space.newbytes(item)
+            elif mytype.typecode == 'u':
+                return space.newunicode(item)
+            assert 0, "unreachable"
 
         # interface
 
@@ -855,29 +1016,16 @@ def make_array(mytype):
             x = self.item_w(w_x)
             index = self.len
             self.setlen(index + 1)
-            self.buffer[index] = x
+            self.get_buffer()[index] = x
+            keepalive_until_here(self)
 
         # List interface
-        def descr_count(self, space, w_val):
-            cnt = 0
-            for i in range(self.len):
-                # XXX jitdriver
-                w_item = self.w_getitem(space, i)
-                if space.is_true(space.eq(w_item, w_val)):
-                    cnt += 1
-            return space.wrap(cnt)
-
-        def descr_index(self, space, w_val):
-            for i in range(self.len):
-                w_item = self.w_getitem(space, i)
-                if space.is_true(space.eq(w_item, w_val)):
-                    return space.wrap(i)
-            raise oefmt(space.w_ValueError, "array.index(x): x not in list")
 
         def descr_reverse(self, space):
-            b = self.buffer
+            b = self.get_buffer()
             for i in range(self.len / 2):
                 b[i], b[self.len - i - 1] = b[self.len - i - 1], b[i]
+            keepalive_until_here(self)
 
         def descr_pop(self, space, i):
             if i < 0:
@@ -885,16 +1033,14 @@ def make_array(mytype):
             if i < 0 or i >= self.len:
                 raise oefmt(space.w_IndexError, "pop index out of range")
             w_val = self.w_getitem(space, i)
+            b = self.get_buffer()
             while i < self.len - 1:
-                self.buffer[i] = self.buffer[i + 1]
+                b[i] = b[i + 1]
                 i += 1
+            keepalive_until_here(self)
             self.setlen(self.len - 1)
             return w_val
 
-        def descr_remove(self, space, w_val):
-            w_idx = self.descr_index(space, w_val)
-            self.descr_pop(space, space.int_w(w_idx))
-
         def descr_insert(self, space, idx, w_val):
             if idx < 0:
                 idx += self.len
@@ -906,10 +1052,12 @@ def make_array(mytype):
             val = self.item_w(w_val)
             self.setlen(self.len + 1)
             i = self.len - 1
+            b = self.get_buffer()
             while i > idx:
-                self.buffer[i] = self.buffer[i - 1]
+                b[i] = b[i - 1]
                 i -= 1
-            self.buffer[i] = val
+            b[i] = val
+            keepalive_until_here(self)
 
         def getitem_slice(self, space, w_idx):
             start, stop, step, size = space.decode_index4(w_idx, self.len)
@@ -917,9 +1065,13 @@ def make_array(mytype):
             w_a.setlen(size, overallocate=False)
             assert step != 0
             j = 0
+            buf = w_a.get_buffer()
+            srcbuf = self.get_buffer()
             for i in range(start, stop, step):
-                w_a.buffer[j] = self.buffer[i]
+                buf[j] = srcbuf[i]
                 j += 1
+            keepalive_until_here(self)
+            keepalive_until_here(w_a)
             return w_a
 
         def setitem(self, space, w_idx, w_item):
@@ -928,7 +1080,8 @@ def make_array(mytype):
                 raise oefmt(self.space.w_TypeError,
                             "can only assign array to array slice")
             item = self.item_w(w_item)
-            self.buffer[idx] = item
+            self.get_buffer()[idx] = item
+            keepalive_until_here(self)
 
         def setitem_slice(self, space, w_idx, w_item):
             if not isinstance(w_item, W_Array):
@@ -945,127 +1098,21 @@ def make_array(mytype):
                 self.fromsequence(w_lst)
             else:
                 j = 0
+                buf = self.get_buffer()
+                srcbuf = w_item.get_buffer()
                 for i in range(start, stop, step):
-                    self.buffer[i] = w_item.buffer[j]
+                    buf[i] = srcbuf[j]
                     j += 1
+                keepalive_until_here(w_item)
+                keepalive_until_here(self)
 
-        def delitem(self, space, i, j):
-            if i < 0:
-                i += self.len
-            if i < 0:
-                i = 0
-            if j < 0:
-                j += self.len
-            if j < 0:
-                j = 0
-            if j > self.len:
-                j = self.len
-            if i >= j:
-                return None
-            oldbuffer = self.buffer
-            self.buffer = lltype.malloc(
-                mytype.arraytype, max(self.len - (j - i), 0), flavor='raw',
-                add_memory_pressure=True)
-            if i:
-                rffi.c_memcpy(
-                    rffi.cast(rffi.VOIDP, self.buffer),
-                    rffi.cast(rffi.VOIDP, oldbuffer),
-                    i * mytype.bytes
-                )
-            if j < self.len:
-                rffi.c_memcpy(
-                    rffi.cast(rffi.VOIDP, rffi.ptradd(self.buffer, i)),
-                    rffi.cast(rffi.VOIDP, rffi.ptradd(oldbuffer, j)),
-                    (self.len - j) * mytype.bytes
-                )
-            self.len -= j - i
-            self.allocated = self.len
-            if oldbuffer:
-                lltype.free(oldbuffer, flavor='raw')
-
-        # Add and mul methods
-        def descr_add(self, space, w_other):
-            if not isinstance(w_other, W_Array):
-                return space.w_NotImplemented
-            a = mytype.w_class(space)
-            a.setlen(self.len + w_other.len, overallocate=False)
-            if self.len:
-                rffi.c_memcpy(
-                    rffi.cast(rffi.VOIDP, a.buffer),
-                    rffi.cast(rffi.VOIDP, self.buffer),
-                    self.len * mytype.bytes
-                )
-            if w_other.len:
-                rffi.c_memcpy(
-                    rffi.cast(rffi.VOIDP, rffi.ptradd(a.buffer, self.len)),
-                    rffi.cast(rffi.VOIDP, w_other.buffer),
-                    w_other.len * mytype.bytes
-                )
-            return a
-
-        def descr_inplace_add(self, space, w_other):
-            if not isinstance(w_other, W_Array):
-                return space.w_NotImplemented
-            oldlen = self.len
-            otherlen = w_other.len
-            self.setlen(oldlen + otherlen)
-            if otherlen:
-                rffi.c_memcpy(
-                    rffi.cast(rffi.VOIDP, rffi.ptradd(self.buffer, oldlen)),
-                    rffi.cast(rffi.VOIDP, w_other.buffer),
-                    otherlen * mytype.bytes
-                )
-            return self
-
-        def descr_mul(self, space, w_repeat):
-            return _mul_helper(space, self, w_repeat, False)
-
-        def descr_inplace_mul(self, space, w_repeat):
-            return _mul_helper(space, self, w_repeat, True)
-
-    def _mul_helper(space, self, w_repeat, is_inplace):
-        try:
-            repeat = space.getindex_w(w_repeat, space.w_OverflowError)
-        except OperationError as e:
-            if e.match(space, space.w_TypeError):
-                return space.w_NotImplemented
-            raise
-        repeat = max(repeat, 0)
-        try:
-            newlen = ovfcheck(self.len * repeat)
-        except OverflowError:
-            raise MemoryError
-        oldlen = self.len
-        if is_inplace:
-            a = self
-            start = 1
-        else:
-            a = mytype.w_class(space)
-            start = 0
-        # <a performance hack>
-        if oldlen == 1:
-            if mytype.unwrap == 'str_w' or mytype.unwrap == 'unicode_w':
-                zero = not ord(self.buffer[0])
-            elif mytype.unwrap == 'int_w' or mytype.unwrap == 'bigint_w':
-                zero = not widen(self.buffer[0])
-            #elif mytype.unwrap == 'float_w':
-            #    value = ...float(self.buffer[0])  xxx handle the case of -0.0
-            else:
-                zero = False
-            if zero:
-                a.setlen(newlen, zero=True, overallocate=False)
-                return a
-            a.setlen(newlen, overallocate=False)
-            item = self.buffer[0]
+        def _repeat_single_item(self, a, start, repeat):
+            # <a performance hack>
+            assert isinstance(a, W_Array)
+            item = self.get_buffer()[0]
+            dstbuf = a.get_buffer()
             for r in range(start, repeat):
-                a.buffer[r] = item
-            return a
-        # </a performance hack>
-        a.setlen(newlen, overallocate=False)
-        for r in range(start, repeat):
-            for i in range(oldlen):
-                a.buffer[r * oldlen + i] = self.buffer[i]
-        return a
+                dstbuf[r] = item
 
     mytype.w_class = W_Array
     W_Array.constructor = W_Array
