@@ -18,11 +18,6 @@ PyAPI_FUNC(long) PyOS_strtol(const char *, char **, int);
 		 PyType_FastSubclass((op)->ob_type, Py_TPFLAGS_LONG_SUBCLASS)
 #define PyLong_CheckExact(op) ((op)->ob_type == &PyLong_Type)
 
-#define PyLong_AS_LONG(op) PyLong_AsLong(op)
-
-#define _PyLong_AsByteArray(v, bytes, n, little_endian, is_signed)   \
-    _PyLong_AsByteArrayO((PyObject *)(v), bytes, n, little_endian, is_signed)
-
 #ifdef __cplusplus
 }
 #endif
