@@ -13,8 +13,7 @@ from pypy.module.cpyext.eval import (
     PyEval_GetBuiltins, PyEval_GetLocals, PyEval_GetGlobals,
     _PyEval_SliceIndex)
 from pypy.module.cpyext.api import (
-    c_fopen, c_fclose, c_fileno, Py_ssize_tP)
-from pypy.module.cpyext.pyobject import get_w_obj_and_decref
+    c_fopen, c_fclose, c_fileno, Py_ssize_tP, is_valid_fd)
 from pypy.interpreter.gateway import interp2app
 from pypy.interpreter.error import OperationError
 from pypy.interpreter.astcompiler import consts
@@ -114,27 +113,25 @@ class TestEval(BaseApiTest):
         w_globals = space.newdict()
         assert 42 * 43 == space.unwrap(
             run("42 * 43", Py_eval_input, w_globals, w_globals))
-        # __builtins__ is added
-        assert PyObject_Size(space, w_globals) == 1
+        assert PyObject_Size(space, w_globals) == 0
 
         assert run("a = 42 * 43", Py_single_input,
                    w_globals, w_globals) == space.w_None
-        py_obj = PyObject_GetItem(space, w_globals, space.wrap("a"))
-        assert 42 * 43 == space.unwrap(get_w_obj_and_decref(space, py_obj))
+        assert 42 * 43 == space.unwrap(
+            PyObject_GetItem(space, w_globals, space.wrap("a")))
 
     def test_run_string_flags(self, space):
         flags = lltype.malloc(PyCompilerFlags, flavor='raw')
         flags.c_cf_flags = rffi.cast(rffi.INT, consts.PyCF_SOURCE_IS_UTF8)
-        flags.c_cf_feature_version = rffi.cast(rffi.INT, -1)
         w_globals = space.newdict()
-        buf = rffi.str2charp("a = 'caf\xc3\xa9'")
+        buf = rffi.str2charp("a = u'caf\xc3\xa9'")
         try:
             PyRun_StringFlags(space, buf, Py_single_input, w_globals,
                               w_globals, flags)
         finally:
             rffi.free_charp(buf)
         w_a = space.getitem(w_globals, space.wrap("a"))
-        assert space.utf8_w(w_a) == u'caf\xe9'.encode("utf-8")
+        assert space.unicode_w(w_a) == u'caf\xe9'
         lltype.free(flags, flavor='raw')
 
     def test_run_file(self, space):
@@ -148,13 +145,12 @@ class TestEval(BaseApiTest):
         c_fclose(fp)
 
         # try again, but with a closed file
-        if self.runappdirect:
-            # according to man 2 fclose, any access of fp is undefined
-            # behaviour. This crashes on some linux systems untranslated
-            fp = c_fopen(str(filepath), "rb")
+        fp = c_fopen(str(filepath), "rb")
+        os.close(c_fileno(fp))
+        with raises_w(space, IOError):
+            PyRun_File(space, fp, filename, Py_file_input, w_globals, w_locals)
+        if is_valid_fd(c_fileno(fp)):
             c_fclose(fp)
-            with raises_w(space, IOError):
-                PyRun_File(space, fp, filename, Py_file_input, w_globals, w_locals)
         rffi.free_charp(filename)
 
     def test_getbuiltins(self, space):
@@ -257,7 +253,7 @@ class AppTestCall(AppTestCpythonExtensionBase):
         module = self.import_extension('foo', [
             ("call_func", "METH_VARARGS",
              """
-                PyObject *t = PyUnicode_FromString("t");
+                PyObject *t = PyString_FromString("t");
                 PyObject *res = PyObject_CallFunctionObjArgs(
                    PyTuple_GetItem(args, 0),
                    Py_None, NULL);
@@ -266,8 +262,8 @@ class AppTestCall(AppTestCpythonExtensionBase):
              """),
             ("call_method", "METH_VARARGS",
              """
-                PyObject *t = PyUnicode_FromString("t");
-                PyObject *count = PyUnicode_FromString("count");
+                PyObject *t = PyString_FromString("t");
+                PyObject *count = PyString_FromString("count");
                 PyObject *res = PyObject_CallMethodObjArgs(
                    PyTuple_GetItem(args, 0),
                    count, t, NULL);
@@ -307,22 +303,21 @@ class AppTestCall(AppTestCpythonExtensionBase):
         mod = module.exec_code(code)
         assert mod.__name__ == "cpyext_test_modname"
         assert mod.__file__ == "someFile"
-        print(dir(mod))
-        print(mod.__dict__)
+        print dir(mod)
+        print mod.__dict__
         assert mod.f(42) == 47
 
         mod = module.exec_code_ex(code)
         assert mod.__name__ == "cpyext_test_modname"
         assert mod.__file__ == "otherFile"
-        print(dir(mod))
-        print(mod.__dict__)
+        print dir(mod)
+        print mod.__dict__
         assert mod.f(42) == 47
 
         # Clean-up
         del sys.modules['cpyext_test_modname']
 
     def test_merge_compiler_flags(self):
-        import sys
         module = self.import_extension('foo', [
             ("get_flags", "METH_NOARGS",
              """
@@ -336,19 +331,11 @@ class AppTestCall(AppTestCpythonExtensionBase):
         assert module.get_flags() == (0, 0)
 
         ns = {'module': module}
-        if not hasattr(sys, 'pypy_version_info'):  # no barry_as_FLUFL on pypy
-            exec("""from __future__ import barry_as_FLUFL    \nif 1:
-                    def nested_flags():
-                        return module.get_flags()""", ns)
-            assert ns['nested_flags']() == (1, 0x40000)
-
-        # the division future should have no effect on Python 3
-        exec("""from __future__ import division    \nif 1:
+        exec """from __future__ import division    \nif 1:
                 def nested_flags():
-                    return module.get_flags()""", ns)
-        assert ns['nested_flags']() == (0, 0)
+                    return module.get_flags()""" in ns
+        assert ns['nested_flags']() == (1, 0x2000)  # CO_FUTURE_DIVISION
 
-    @pytest.mark.xfail("'linux' not in sys.platform", reason='Hangs the process', run=False)
     def test_recursive_function(self):
         module = self.import_extension('foo', [
             ("call_recursive", "METH_NOARGS",
@@ -374,11 +361,14 @@ class AppTestCall(AppTestCpythonExtensionBase):
                 };
              '''
             )
-        excinfo = raises(RecursionError, module.call_recursive)
-        assert 'while calling recurse' in str(excinfo.value)
+        try:
+            res = module.call_recursive()
+        except RuntimeError as e:
+            assert 'while calling recurse' in str(e)
+        else:
+            assert False, "expected RuntimeError"
 
     def test_build_class(self):
-            """
             # make sure PyObject_Call generates a proper PyTypeObject,
             # along the way verify that userslot has iter and next
             module = self.import_extension('foo', [
@@ -403,35 +393,7 @@ class AppTestCall(AppTestCpythonExtensionBase):
                         return NULL;
                     }
                     return args->ob_type->tp_iternext(args);
-                 '''),
-                ('await_', "METH_O",
-                 '''
-                    if (NULL == args->ob_type->tp_as_async->am_await)
-                    {
-                        PyErr_SetString(PyExc_TypeError, "NULL am_await");
-                        return NULL;
-                    }
-                    return args->ob_type->tp_as_async->am_await(args);
-                 '''),
-                ('aiter', "METH_O",
-                 '''
-                    if (NULL == args->ob_type->tp_as_async->am_aiter)
-                    {
-                        PyErr_SetString(PyExc_TypeError, "NULL am_aiter");
-                        return NULL;
-                    }
-                    return args->ob_type->tp_as_async->am_aiter(args);
-                 '''),
-                ('anext', "METH_O",
-                 '''
-                    if (NULL == args->ob_type->tp_as_async->am_anext)
-                    {
-                        PyErr_SetString(PyExc_TypeError, "NULL am_anext");
-                        return NULL;
-                    }
-                    return args->ob_type->tp_as_async->am_anext(args);
-                 '''),
-            ])
+                 '''),])
             def __init__(self, N):
                 self.N = N
                 self.i = 0
@@ -447,7 +409,7 @@ class AppTestCall(AppTestCpythonExtensionBase):
                 raise StopIteration
 
             d = {'__init__': __init__, '__iter__': __iter__, 'next': __next__,
-                 '__next__': __next__}
+                 '__next__': next}
             C = module.object_call(('Iterable', (object,), d))
             c = C(5)
             i = module.iter(c)
@@ -458,71 +420,3 @@ class AppTestCall(AppTestCpythonExtensionBase):
             except StopIteration:
                 pass
             assert out == [0, 1, 2, 3, 4]
-
-            def run_async(coro):
-                buffer = []
-                result = None
-                while True:
-                    try:
-                        buffer.append(coro.send(None))
-                    except StopIteration as ex:
-                        result = ex.value
-                        break
-                return buffer, result
-
-            def __await__(self):
-                yield 42
-                return 100
-
-            Awaitable = module.object_call((
-                'Awaitable', (object,), {'__await__': __await__}))
-
-            async def wrapper():
-                return await Awaitable()
-
-            assert run_async(module.await_(Awaitable())) == ([42], 100)
-            assert run_async(wrapper()) == ([42], 100)
-
-            def __aiter__(self):
-                return self
-
-            async def __anext__(self):
-                if self.i < self.N:
-                    res = self.i
-                    self.i += 1
-                    return res
-                raise StopAsyncIteration
-
-            AIter = module.object_call(('AIter', (object,),
-                {'__init__': __init__, '__aiter__': __aiter__,
-                 '__anext__': __anext__}))
-
-            async def list1():
-                s = []
-                async for i in AIter(3):
-                    s.append(i)
-                return s
-            async def list2():
-                s = []
-                ait = module.aiter(AIter(3))
-                try:
-                    while True:
-                        s.append(await module.anext(ait))
-                except StopAsyncIteration:
-                    return s
-
-            assert run_async(list1()) == ([], [0, 1, 2])
-            assert run_async(list2()) == ([], [0, 1, 2])
-            """
-
-    def test_getframe(self):
-        import sys
-        module = self.import_extension('foo', [
-            ("getframe1", "METH_NOARGS",
-             """
-                PyFrameObject *x = PyEval_GetFrame();
-                Py_INCREF(x);
-                return (PyObject *)x;
-             """),], prologue="#include <frameobject.h>\n")
-        res = module.getframe1()
-        assert res is sys._getframe(0)
