@@ -1,7 +1,6 @@
 import pytest
 import os
 from pypy.interpreter.error import OperationError
-from pypy.module.cpyext.pyobject import make_ref, decref
 from pypy.module.cpyext.test.test_api import BaseApiTest
 from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
 from rpython.rtyper.lltypesystem import rffi, lltype
@@ -11,8 +10,6 @@ import pypy.module.micronumpy.constants as NPY
 from pypy.module.cpyext.ndarrayobject import (
     _PyArray_FromAny, _PyArray_FromObject)
 
-pytest.skip("Micronumpy not yet supported on py3k.")
-
 def scalar(space):
     dtype = get_dtype_cache(space).w_float64dtype
     return W_NDimArray.new_scalar(space, dtype, space.wrap(10.))
@@ -234,13 +231,6 @@ class TestNDArrayObject(BaseApiTest):
                         signature)
         '''
 
-    def test_ndarray_ref(self, space, api):
-        w_obj = space.appexec([], """():
-            import _numpypy
-            return _numpypy.multiarray.dtype('int64').type(2)""")
-        ref = make_ref(space, w_obj)
-        decref(space, ref)
-
 class AppTestNDArray(AppTestCpythonExtensionBase):
 
     def setup_class(cls):
