@@ -2,16 +2,16 @@ import pytest
 
 from pypy.module.cpyext.test.test_api import BaseApiTest, raises_w
 from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
-from rpython.rtyper.lltypesystem import rffi
-from pypy.module.cpyext.pyobject import get_w_obj_and_decref
+from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.module.cpyext.api import (
-    Py_LT, Py_LE, Py_NE, Py_EQ, Py_GE, Py_GT, INTP_real)
+    Py_LT, Py_LE, Py_NE, Py_EQ, Py_GE, Py_GT)
 from pypy.module.cpyext.object import (
     PyObject_IsTrue, PyObject_Not, PyObject_GetAttrString,
     PyObject_DelAttrString, PyObject_GetAttr, PyObject_DelAttr,
     PyObject_GetItem,
     PyObject_IsInstance, PyObject_IsSubclass, PyObject_AsFileDescriptor,
-    PyObject_Hash)
+    PyObject_Hash, PyObject_Cmp, PyObject_Unicode
+)
 
 class TestObject(BaseApiTest):
     def test_IsTrue(self, space, api):
@@ -28,7 +28,7 @@ class TestObject(BaseApiTest):
     def test_exception(self, space, api):
         w_obj = space.appexec([], """():
             class C:
-                def __bool__(self):
+                def __nonzero__(self):
                     raise ValueError
             return C()""")
 
@@ -70,8 +70,7 @@ class TestObject(BaseApiTest):
     def test_getattr(self, space):
         charp1 = rffi.str2charp("__len__")
         charp2 = rffi.str2charp("not_real")
-        assert get_w_obj_and_decref(space,
-            PyObject_GetAttrString(space, space.wrap(""), charp1))
+        assert PyObject_GetAttrString(space, space.wrap(""), charp1)
 
         with raises_w(space, AttributeError):
             PyObject_GetAttrString(space, space.wrap(""), charp2)
@@ -80,20 +79,17 @@ class TestObject(BaseApiTest):
         rffi.free_charp(charp1)
         rffi.free_charp(charp2)
 
-        assert get_w_obj_and_decref(space,
-            PyObject_GetAttr(space, space.wrap(""), space.wrap("__len__")))
+        assert PyObject_GetAttr(space, space.wrap(""), space.wrap("__len__"))
         with raises_w(space, AttributeError):
             PyObject_DelAttr(space, space.wrap(""), space.wrap("__len__"))
 
     def test_getitem(self, space, api):
         w_t = space.wrap((1, 2, 3, 4, 5))
-        assert space.unwrap(get_w_obj_and_decref(space,
-            api.PyObject_GetItem(w_t, space.wrap(3)))) == 4
+        assert space.unwrap(api.PyObject_GetItem(w_t, space.wrap(3))) == 4
 
         w_d = space.newdict()
         space.setitem(w_d, space.wrap("a key!"), space.wrap(72))
-        assert space.unwrap(get_w_obj_and_decref(space,
-            api.PyObject_GetItem(w_d, space.wrap("a key!")))) == 72
+        assert space.unwrap(api.PyObject_GetItem(w_d, space.wrap("a key!"))) == 72
 
         assert api.PyObject_SetItem(w_d, space.wrap("key"), space.w_None) == 0
         assert space.getitem(w_d, space.wrap("key")) is space.w_None
@@ -107,15 +103,15 @@ class TestObject(BaseApiTest):
 
     def test_str(self, space, api):
         w_list = space.newlist([space.w_None, space.wrap(42)])
-        assert space.text_w(api.PyObject_Str(None)) == "<NULL>"
-        assert space.text_w(api.PyObject_Str(w_list)) == "[None, 42]"
-        assert space.text_w(api.PyObject_Str(space.wrap("a"))) == "a"
+        assert space.str_w(api.PyObject_Str(None)) == "<NULL>"
+        assert space.str_w(api.PyObject_Str(w_list)) == "[None, 42]"
+        assert space.str_w(api.PyObject_Str(space.wrap("a"))) == "a"
 
     def test_repr(self, space, api):
         w_list = space.newlist([space.w_None, space.wrap(42)])
-        assert space.text_w(api.PyObject_Repr(None)) == "<NULL>"
-        assert space.text_w(api.PyObject_Repr(w_list)) == "[None, 42]"
-        assert space.text_w(api.PyObject_Repr(space.wrap("a"))) == "'a'"
+        assert space.str_w(api.PyObject_Repr(None)) == "<NULL>"
+        assert space.str_w(api.PyObject_Repr(w_list)) == "[None, 42]"
+        assert space.str_w(api.PyObject_Repr(space.wrap("a"))) == "'a'"
 
     def test_RichCompare(self, space, api):
         def compare(w_o1, w_o2, opid):
@@ -197,6 +193,28 @@ class TestObject(BaseApiTest):
     def test_type(self, space, api):
         assert api.PyObject_Type(space.wrap(72)) is space.w_int
 
+    def test_compare(self, space, api):
+        assert api.PyObject_Compare(space.wrap(42), space.wrap(72)) == -1
+        assert api.PyObject_Compare(space.wrap(72), space.wrap(42)) == 1
+        assert api.PyObject_Compare(space.wrap("a"), space.wrap("a")) == 0
+
+    def test_cmp(self, space, api):
+        w = space.wrap
+        with lltype.scoped_alloc(rffi.INTP.TO, 1) as ptr:
+            assert api.PyObject_Cmp(w(42), w(72), ptr) == 0
+            assert ptr[0] == -1
+            assert api.PyObject_Cmp(w("a"), w("a"), ptr) == 0
+            assert ptr[0] == 0
+            with raises_w(space, UnicodeDecodeError):
+                PyObject_Cmp(space, w(u"\xe9"), w("\xe9"), ptr)
+
+    def test_unicode(self, space, api):
+        assert space.unicode_w(api.PyObject_Unicode(None)) == u"<NULL>"
+        assert space.unicode_w(api.PyObject_Unicode(space.wrap([]))) == u"[]"
+        assert space.unicode_w(api.PyObject_Unicode(space.wrap("e"))) == u"e"
+        with raises_w(space, UnicodeDecodeError):
+            PyObject_Unicode(space, space.wrap("\xe9"))
+
     def test_dir(self, space, api):
         w_dir = api.PyObject_Dir(space.sys)
         assert space.isinstance_w(w_dir, space.w_list)
@@ -204,7 +222,7 @@ class TestObject(BaseApiTest):
 
     def test_format(self, space, api):
         w_int = space.wrap(42)
-        fmt = space.text_w(api.PyObject_Format(w_int, space.wrap('#b')))
+        fmt = space.str_w(api.PyObject_Format(w_int, space.wrap('#b')))
         assert fmt == '0b101010'
 
 class AppTestObject(AppTestCpythonExtensionBase):
@@ -214,11 +232,14 @@ class AppTestObject(AppTestCpythonExtensionBase):
 
         AppTestCpythonExtensionBase.setup_class.im_func(cls)
         tmpname = str(pytest.ensuretemp('out', dir=0))
-        cls.w_tmpname = cls.space.wrap(tmpname)
+        if cls.runappdirect:
+            cls.tmpname = tmpname
+        else:
+            cls.w_tmpname = cls.space.wrap(tmpname)
 
         if not cls.runappdirect:
             cls.total_mem = 0
-            def add_memory_pressure(estimate, object=None):
+            def add_memory_pressure(estimate):
                 assert estimate >= 0
                 cls.total_mem += estimate
             cls.orig_add_memory_pressure = [rgc.add_memory_pressure]
@@ -235,7 +256,7 @@ class AppTestObject(AppTestCpythonExtensionBase):
                 gateway.interp2app(_cur_memory_pressure))
         else:
             def _skip_test(*ignored):
-                skip("not for -A testing")
+                pytest.skip("not for -A testing")
             cls.w_reset_memory_pressure = _skip_test
 
     def teardown_class(cls):
@@ -247,29 +268,15 @@ class AppTestObject(AppTestCpythonExtensionBase):
         module = self.import_extension('foo', [
             ("malloctest", "METH_NOARGS",
              """
-                 PyObject *obj = PyObject_MALLOC(sizeof(PyFloatObject));
-                 obj = PyObject_Init(obj, &PyFloat_Type);
+                 PyObject *obj = PyObject_MALLOC(sizeof(PyIntObject));
+                 obj = PyObject_Init(obj, &PyInt_Type);
                  if (obj != NULL)
-                     ((PyFloatObject *)obj)->ob_fval = -12.34;
+                     ((PyIntObject *)obj)->ob_ival = -424344;
                  return obj;
              """)])
         x = module.malloctest()
-        assert type(x) is float
-        assert x == -12.34
-
-    def test_object_calloc(self):
-        module = self.import_extension('foo', [
-            ("calloctest", "METH_NOARGS",
-             """
-                 PyObject *obj = PyObject_Calloc(1, sizeof(PyFloatObject));
-                 if (obj == NULL)
-                    return NULL;
-                 obj = PyObject_Init(obj, &PyFloat_Type);
-                 return obj;
-             """)])
-        x = module.calloctest()
-        assert type(x) is float
-        assert x == 0.0
+        assert type(x) is int
+        assert x == -424344
 
     def test_object_realloc(self):
         if not self.runappdirect:
@@ -284,12 +291,12 @@ class AppTestObject(AppTestCpythonExtensionBase):
                  /* realloc() takes care of freeing orig, if changed */
                  if (copy == NULL)
                      Py_RETURN_NONE;
-                 ret = PyBytes_FromStringAndSize(copy, 12);
+                 ret = PyString_FromStringAndSize(copy, 12);
                  PyObject_Free(copy);
                  return ret;
              """)])
         x = module.realloctest()
-        assert x == b'hello world\x00'
+        assert x == 'hello world\x00'
 
     def test_TypeCheck(self):
         module = self.import_extension('foo', [
@@ -302,6 +309,7 @@ class AppTestObject(AppTestCpythonExtensionBase):
         assert module.typecheck(1, int)
         assert module.typecheck('foo', str)
         assert module.typecheck('foo', object)
+        assert module.typecheck(1L, long)
         assert module.typecheck(True, bool)
         assert module.typecheck(1.2, float)
         assert module.typecheck(int, type)
@@ -313,7 +321,7 @@ class AppTestObject(AppTestCpythonExtensionBase):
                  PyObject *fname = PyTuple_GetItem(args, 0);
                  PyObject *obj = PyTuple_GetItem(args, 1);
 
-                 FILE *fp = fopen(_PyUnicode_AsString(fname), "wb");
+                 FILE *fp = fopen(PyString_AsString(fname), "wb");
                  int ret;
                  if (fp == NULL)
                      Py_RETURN_NONE;
@@ -333,7 +341,7 @@ class AppTestObject(AppTestCpythonExtensionBase):
         assert module.dump(self.tmpname, None)
         assert open(self.tmpname).read() == 'None<nil>'
 
-    def test_isinstance(self):
+    def test_issue1970(self):
         module = self.import_extension('foo', [
             ("ismapping", "METH_O",
              """
@@ -349,22 +357,10 @@ class AppTestObject(AppTestCpythonExtensionBase):
                      Py_DECREF(mapping_t);
                      Py_RETURN_FALSE;
                  }
-             """),
-            ("is_function", "METH_O",
-             """
-                if (PyObject_IsInstance(args, (PyObject *)&PyFunction_Type)) {
-                     Py_RETURN_TRUE;
-                } else {
-                     Py_RETURN_FALSE;
-                }
-             """),])
+             """)])
         import collections
         assert isinstance(dict(), collections.Mapping)
         assert module.ismapping(dict())
-        def f():
-            pass
-        assert not module.is_function(1)
-        assert module.is_function(f)
 
     def test_format_returns_unicode(self):
         module = self.import_extension('foo', [
@@ -375,51 +371,14 @@ class AppTestObject(AppTestCpythonExtensionBase):
                 return obj;
             """)])
         a = module.empty_format('hello')
-        assert isinstance(a, str)
-        a = module.empty_format(type('hello'))
-        assert isinstance(a, str)
-
-    def test_Bytes(self):
-        class sub1(bytes):
-            pass
-        class sub2(bytes):
-            def __bytes__(self):
-                return self
-        module = self.import_extension('test_Bytes', [
-            ('asbytes', 'METH_O',
-             """
-                return PyObject_Bytes(args);
-             """)])
-        assert type(module.asbytes(sub1(b''))) is bytes
-        assert type(module.asbytes(sub2(b''))) is sub2
-
-    def test_LengthHint(self):
-        import operator
-        class WithLen:
-            def __len__(self):
-                return 1
-            def __length_hint__(self):
-                return 42
-        class NoLen:
-            def __length_hint__(self):
-                return 2
-        module = self.import_extension('test_LengthHint', [
-            ('length_hint', 'METH_VARARGS',
-             """
-                 PyObject *obj = PyTuple_GET_ITEM(args, 0);
-                 Py_ssize_t i = PyLong_AsSsize_t(PyTuple_GET_ITEM(args, 1));
-                 return PyLong_FromSsize_t(PyObject_LengthHint(obj, i));
-             """)])
-        assert module.length_hint(WithLen(), 5) == operator.length_hint(WithLen(), 5) == 1
-        assert module.length_hint(NoLen(), 5) == operator.length_hint(NoLen(), 5) == 2
-        assert module.length_hint(object(), 5) == operator.length_hint(object(), 5) == 5
+        assert isinstance(a, unicode)
 
     def test_add_memory_pressure(self):
         self.reset_memory_pressure()    # for the potential skip
         module = self.import_extension('foo', [
             ("foo", "METH_O",
             """
-                PyTraceMalloc_Track(0, 0, PyLong_AsLong(args) - sizeof(long));
+                _PyTraceMalloc_Track(0, 0, PyInt_AsLong(args) - sizeof(long));
                 Py_INCREF(Py_None);
                 return Py_None;
             """)])
@@ -439,72 +398,6 @@ class AppTestObject(AppTestCpythonExtensionBase):
         module.foo(35000)
         assert self.cur_memory_pressure() == 65536 + 80000 + 70000
 
-    def test_repr_enter_leave(self):
-        module = self.import_extension('foo', [
-            ("enter", "METH_O",
-            """
-                return PyLong_FromLong(Py_ReprEnter(args));
-            """),
-            ("leave", "METH_O",
-            """
-                Py_ReprLeave(args);
-                Py_INCREF(Py_None);
-                return Py_None;
-            """)])
-        obj1 = [42]
-        obj2 = [42]   # another list
-
-        n = module.enter(obj1)
-        assert n == 0
-        module.leave(obj1)
-
-        n = module.enter(obj1)
-        assert n == 0
-        n = module.enter(obj1)
-        assert n == 1
-        n = module.enter(obj1)
-        assert n == 1
-        module.leave(obj1)
-
-        n = module.enter(obj1)
-        assert n == 0
-        n = module.enter(obj2)
-        assert n == 0
-        n = module.enter(obj1)
-        assert n == 1
-        n = module.enter(obj2)
-        assert n == 1
-        module.leave(obj1)
-        n = module.enter(obj2)
-        assert n == 1
-        module.leave(obj2)
-
-    def test_GenericGetSetDict(self):
-        module = self.import_extension('test_GenericGetSetDict', [
-            ('test1', 'METH_VARARGS',
-             """
-                 PyObject *obj = PyTuple_GET_ITEM(args, 0);
-                 PyObject *newdict = PyTuple_GET_ITEM(args, 1);
-
-                 PyObject *olddict = PyObject_GenericGetDict(obj, NULL);
-                 if (olddict == NULL)
-                    return NULL;
-                 int res = PyObject_GenericSetDict(obj, newdict, NULL);
-                 if (res != 0)
-                     return NULL;
-                 return olddict;
-             """)])
-        class A:
-            pass
-        a = A()
-        a.x = 42
-        nd = {'y': 43}
-        d = module.test1(a, nd)
-        assert d == {'x': 42}
-        assert a.y == 43
-        assert a.__dict__ is nd
-
-
 class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
     """
     PyBuffer_FillInfo populates the fields of a Py_buffer from its arguments.
@@ -518,16 +411,16 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
                 ("fillinfo", "METH_NOARGS",
                  """
     Py_buffer buf;
-    PyObject *str = PyBytes_FromString("hello, world.");
+    PyObject *str = PyString_FromString("hello, world.");
     PyObject *result;
 
-    if (PyBuffer_FillInfo(&buf, NULL, PyBytes_AsString(str), 13, 0, 0)) {
+    if (PyBuffer_FillInfo(&buf, NULL, PyString_AsString(str), 13, 0, 0)) {
         return NULL;
     }
 
     /* Check a few things we want to have happened.
      */
-    if (buf.buf != PyBytes_AsString(str)) {
+    if (buf.buf != PyString_AsString(str)) {
         PyErr_SetString(PyExc_ValueError, "buf field not initialized");
         return NULL;
     }
@@ -545,7 +438,7 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
     /* Give back a new string to the caller, constructed from data in the
      * Py_buffer.
      */
-    if (!(result = PyBytes_FromStringAndSize(buf.buf, buf.len))) {
+    if (!(result = PyString_FromStringAndSize(buf.buf, buf.len))) {
         return NULL;
     }
 
@@ -557,7 +450,7 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
     return result;
                  """)])
         result = module.fillinfo()
-        assert b"hello, world." == result
+        assert "hello, world." == result
 
 
     def test_fillWithObject(self):
@@ -570,10 +463,10 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
                 ("fillinfo", "METH_NOARGS",
                  """
     Py_buffer buf;
-    PyObject *str = PyBytes_FromString("hello, world.");
+    PyObject *str = PyString_FromString("hello, world.");
     PyObject *result;
 
-    if (PyBuffer_FillInfo(&buf, str, PyBytes_AsString(str), 13, 0, 0)) {
+    if (PyBuffer_FillInfo(&buf, str, PyString_AsString(str), 13, 0, 0)) {
         return NULL;
     }
 
@@ -585,7 +478,7 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
     /* Give back a new string to the caller, constructed from data in the
      * Py_buffer.  It better still be valid.
      */
-    if (!(result = PyBytes_FromStringAndSize(buf.buf, buf.len))) {
+    if (!(result = PyString_FromStringAndSize(buf.buf, buf.len))) {
         return NULL;
     }
 
@@ -605,7 +498,7 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
     return result;
                  """)])
         result = module.fillinfo()
-        assert b"hello, world." == result
+        assert "hello, world." == result
 
 
     def test_fillReadonly(self):
@@ -616,9 +509,9 @@ class AppTestPyBuffer_FillInfo(AppTestCpythonExtensionBase):
                 ("fillinfo", "METH_NOARGS",
                  """
     Py_buffer buf;
-    PyObject *str = PyBytes_FromString("hello, world.");
+    PyObject *str = PyString_FromString("hello, world.");
 
-    if (PyBuffer_FillInfo(&buf, str, PyBytes_AsString(str), 13,
+    if (PyBuffer_FillInfo(&buf, str, PyString_AsString(str), 13,
                           1, PyBUF_WRITABLE)) {
         Py_DECREF(str);
         return NULL;
@@ -643,9 +536,9 @@ class AppTestPyBuffer_Release(AppTestCpythonExtensionBase):
                 ("release", "METH_NOARGS",
                  """
     Py_buffer buf;
-    buf.obj = PyBytes_FromString("release me!");
-    buf.buf = PyBytes_AsString(buf.obj);
-    buf.len = PyBytes_Size(buf.obj);
+    buf.obj = PyString_FromString("release me!");
+    buf.buf = PyString_AsString(buf.obj);
+    buf.len = PyString_Size(buf.obj);
 
     /* The Py_buffer owns the only reference to that string.  Release the
      * Py_buffer and the string should be released as well.
