@@ -1,9 +1,12 @@
-from rpython.rtyper.lltypesystem import rffi, lltype
+from rpython.rtyper.lltypesystem import rffi
 from pypy.interpreter.error import OperationError
-from pypy.module.cpyext.test.test_api import BaseApiTest
+from pypy.module.cpyext.test.test_api import BaseApiTest, raises_w
 from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
-from pypy.module.cpyext import sequence
-import py.test
+from pypy.module.cpyext.sequence import (
+    PySequence_Fast, PySequence_Contains, PySequence_Index,
+    PySequence_GetItem, PySequence_SetItem, PySequence_DelItem)
+
+import pytest
 
 class TestSequence(BaseApiTest):
     def test_check(self, space, api):
@@ -56,16 +59,12 @@ class TestSequence(BaseApiTest):
         w_t2 = api.PySequence_InPlaceRepeat(w_t1, 3)
         assert space.unwrap(w_t2) == [0, 1, 0, 1, 0, 1]
 
-    def test_exception(self, space, api):
+    def test_exception(self, space):
         message = rffi.str2charp("message")
-        assert not api.PySequence_Fast(space.wrap(3), message)
-        assert api.PyErr_Occurred() is space.w_TypeError
-        api.PyErr_Clear()
-
-        exc = raises(OperationError, sequence.PySequence_Fast,
-                     space, space.wrap(3), message)
-        assert exc.value.match(space, space.w_TypeError)
-        assert space.str_w(exc.value.get_w_value(space)) == "message"
+        with pytest.raises(OperationError) as excinfo:
+            PySequence_Fast(space, space.wrap(3), message)
+        assert excinfo.value.match(space, space.w_TypeError)
+        assert space.str_w(excinfo.value.get_w_value(space)) == "message"
         rffi.free_charp(message)
 
     def test_get_slice(self, space, api):
@@ -80,7 +79,7 @@ class TestSequence(BaseApiTest):
 
     def test_get_slice_fast(self, space, api):
         w_t = space.wrap([1, 2, 3, 4, 5])
-        api.PySequence_Fast(w_t, "foo") # converts
+        api.PySequence_Fast(w_t, "foo")  # converts
         assert space.unwrap(api.PySequence_GetSlice(w_t, 2, 4)) == [3, 4]
         assert space.unwrap(api.PySequence_GetSlice(w_t, 1, -1)) == [2, 3, 4]
 
@@ -97,13 +96,12 @@ class TestSequence(BaseApiTest):
         exc = raises(OperationError, space.next, w_iter)
         assert exc.value.match(space, space.w_StopIteration)
 
-    def test_contains(self, space, api):
+    def test_contains(self, space):
         w_t = space.wrap((1, 'ha'))
-        assert api.PySequence_Contains(w_t, space.wrap(u'ha'))
-        assert not api.PySequence_Contains(w_t, space.wrap(2))
-        assert api.PySequence_Contains(space.w_None, space.wrap(2)) == -1
-        assert api.PyErr_Occurred()
-        api.PyErr_Clear()
+        assert PySequence_Contains(space, w_t, space.wrap(u'ha'))
+        assert not PySequence_Contains(space, w_t, space.wrap(2))
+        with raises_w(space, TypeError):
+            PySequence_Contains(space, space.w_None, space.wrap(2))
 
     def test_setitem(self, space, api):
         w_value = space.wrap(42)
@@ -112,39 +110,33 @@ class TestSequence(BaseApiTest):
         result = api.PySequence_SetItem(l, 0, w_value)
         assert result != -1
         assert space.eq_w(space.getitem(l, space.wrap(0)), w_value)
-
-        self.raises(space, api, IndexError, api.PySequence_SetItem,
-                    l, 3, w_value)
+        with raises_w(space, IndexError):
+            PySequence_SetItem(space, l, 3, w_value)
 
         t = api.PyTuple_New(1)
         api.PyTuple_SetItem(t, 0, l)
-        self.raises(space, api, TypeError, api.PySequence_SetItem,
-                    t, 0, w_value)
-
-        self.raises(space, api, TypeError, api.PySequence_SetItem,
-                    space.newdict(), 0, w_value)
+        with raises_w(space, TypeError):
+            PySequence_SetItem(space, t, 0, w_value)
+        with raises_w(space, TypeError):
+            PySequence_SetItem(space, space.newdict(), 0, w_value)
 
     def test_delitem(self, space, api):
         w_l = space.wrap([1, 2, 3, 4])
-
         result = api.PySequence_DelItem(w_l, 2)
         assert result == 0
         assert space.eq_w(w_l, space.wrap([1, 2, 4]))
-
-        self.raises(space, api, IndexError, api.PySequence_DelItem,
-                    w_l, 3)
+        with raises_w(space, IndexError):
+            PySequence_DelItem(space, w_l, 3)
 
     def test_getitem(self, space, api):
         thelist = [8, 7, 6, 5, 4, 3, 2, 1]
         w_l = space.wrap(thelist)
-
         result = api.PySequence_GetItem(w_l, 4)
         assert space.is_true(space.eq(result, space.wrap(4)))
-
         result = api.PySequence_ITEM(w_l, 4)
         assert space.is_true(space.eq(result, space.wrap(4)))
-
-        self.raises(space, api, IndexError, api.PySequence_GetItem, w_l, 9000)
+        with raises_w(space, IndexError):
+            PySequence_GetItem(space, w_l, 9000)
 
     def test_index(self, space, api):
         thelist = [9, 8, 7, 6, 5, 4, 3, 2, 1]
@@ -155,10 +147,8 @@ class TestSequence(BaseApiTest):
         assert result == thelist.index(5)
 
         w_tofind = space.wrap(9001)
-        result = api.PySequence_Index(w_l, w_tofind)
-        assert result == -1
-        assert api.PyErr_Occurred() is space.w_ValueError
-        api.PyErr_Clear()
+        with raises_w(space, ValueError):
+            PySequence_Index(space, w_l, w_tofind)
 
         w_gen = space.appexec([], """():
            return (x ** 2 for x in range(40))""")
@@ -196,7 +186,7 @@ class TestCPyListStrategy(BaseApiTest):
         assert space.int_w(space.len(w_l)) == 4
         assert space.int_w(space.getitem(w_l, space.wrap(1))) == 2
         assert space.int_w(space.getitem(w_l, space.wrap(0))) == 1
-        e = py.test.raises(OperationError, space.getitem, w_l, space.wrap(15))
+        e = pytest.raises(OperationError, space.getitem, w_l, space.wrap(15))
         assert "list index out of range" in e.value.errorstr(space)
         assert space.int_w(space.getitem(w_l, space.wrap(-1))) == 4
         space.setitem(w_l, space.wrap(1), space.wrap(13))
@@ -236,6 +226,15 @@ class TestCPyListStrategy(BaseApiTest):
         w_l.inplace_mul(2)
         assert space.int_w(space.len(w_l)) == 10
 
+    def test_getstorage_copy(self, space, api):
+        w = space.wrap
+        w_l = w([1, 2, 3, 4])
+        api.PySequence_Fast(w_l, "foo") # converts
+
+        w_l1 = w([])
+        space.setitem(w_l1, space.newslice(w(0), w(0), w(1)), w_l)
+        assert map(space.unwrap, space.unpackiterable(w_l1)) == [1, 2, 3, 4]
+
 
 class AppTestSequenceObject(AppTestCpythonExtensionBase):
     def test_fast(self):
