@@ -4,7 +4,7 @@ from pypy.interpreter.typedef import (TypeDef, GetSetProperty,
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.gateway import unwrap_spec, interp2app
 from pypy.interpreter.pycode import PyCode
-from pypy.interpreter.error import OperationError
+from pypy.interpreter.error import oefmt
 from rpython.rtyper.lltypesystem import lltype
 from rpython.rtyper.annlowlevel import cast_base_ptr_to_instance, hlstr
 from rpython.rtyper.rclass import OBJECT
@@ -39,10 +39,10 @@ def wrap_greenkey(space, jitdriver, greenkey, greenkey_repr):
         ll_code = lltype.cast_opaque_ptr(lltype.Ptr(OBJECT),
                                          greenkey[2].getref_base())
         pycode = cast_base_ptr_to_instance(PyCode, ll_code)
-        return space.newtuple([space.wrap(pycode), space.wrap(next_instr),
+        return space.newtuple([pycode, space.newint(next_instr),
                                space.newbool(bool(is_being_profiled))])
     else:
-        return space.wrap(greenkey_repr)
+        return space.newtext(greenkey_repr)
 
 @unwrap_spec(operations=bool)
 def set_compile_hook(space, w_hook, operations=True):
@@ -127,15 +127,15 @@ def wrap_oplist(space, logops, operations, ops_offset=None):
             l_w.append(WrappedOp(name, ofs, logops.repr_of_resop(op)))
     return l_w
 
-@unwrap_spec(offset=int, repr=str, name=str)
+@unwrap_spec(offset=int, repr='text', name='text')
 def descr_new_resop(space, w_tp, name, offset=-1, repr=''):
     return WrappedOp(name, offset, repr)
 
-@unwrap_spec(offset=int, repr=str, name=str, hash=r_uint)
+@unwrap_spec(offset=int, repr='text', name='text', hash=r_uint)
 def descr_new_guardop(space, w_tp, name, offset=-1, repr='', hash=r_uint(0)):
     return GuardOp(name, offset, repr, hash)
 
-@unwrap_spec(repr=str, name=str, jd_name=str, call_depth=int, call_id=int)
+@unwrap_spec(repr='text', name='text', jd_name='text', call_depth=int, call_id=int)
 def descr_new_dmp(space, w_tp, name, repr, jd_name, call_depth, call_id,
     w_greenkey):
 
@@ -152,10 +152,10 @@ class WrappedOp(W_Root):
         self.repr_of_resop = repr_of_resop
 
     def descr_repr(self, space):
-        return space.wrap(self.repr_of_resop)
+        return space.newtext(self.repr_of_resop)
 
     def descr_name(self, space):
-        return space.wrap(self.name)
+        return space.newtext(self.name)
 
 class GuardOp(WrappedOp):
     def __init__(self, name, offset, repr_of_resop, hash):
@@ -178,16 +178,20 @@ class DebugMergePoint(WrappedOp):
 
     def get_pycode(self, space):
         if self.jd_name == pypyjitdriver.name:
-            return space.getitem(self.w_greenkey, space.wrap(0))
-        raise OperationError(space.w_AttributeError, space.wrap("This DebugMergePoint doesn't belong to the main Python JitDriver"))
+            return space.getitem(self.w_greenkey, space.newint(0))
+        raise oefmt(space.w_AttributeError,
+                    "This DebugMergePoint doesn't belong to the main Python "
+                    "JitDriver")
 
     def get_bytecode_no(self, space):
         if self.jd_name == pypyjitdriver.name:
-            return space.getitem(self.w_greenkey, space.wrap(1))
-        raise OperationError(space.w_AttributeError, space.wrap("This DebugMergePoint doesn't belong to the main Python JitDriver"))
+            return space.getitem(self.w_greenkey, space.newint(1))
+        raise oefmt(space.w_AttributeError,
+                    "This DebugMergePoint doesn't belong to the main Python "
+                    "JitDriver")
 
     def get_jitdriver_name(self, space):
-        return space.wrap(self.jd_name)
+        return space.newtext(self.jd_name)
 
 WrappedOp.typedef = TypeDef(
     'ResOperation',
@@ -195,7 +199,7 @@ WrappedOp.typedef = TypeDef(
     __new__ = interp2app(descr_new_resop),
     __repr__ = interp2app(WrappedOp.descr_repr),
     name = GetSetProperty(WrappedOp.descr_name),
-    offset = interp_attrproperty("offset", cls=WrappedOp),
+    offset = interp_attrproperty("offset", cls=WrappedOp, wrapfn="newint"),
 )
 WrappedOp.typedef.acceptable_as_base_class = False
 
@@ -205,8 +209,8 @@ GuardOp.typedef = TypeDef(
     __new__ = interp2app(descr_new_guardop),
     __repr__ = interp2app(GuardOp.descr_repr),
     name = GetSetProperty(GuardOp.descr_name),
-    offset = interp_attrproperty("offset", cls=GuardOp),
-    hash = interp_attrproperty("hash", cls=GuardOp),
+    offset = interp_attrproperty("offset", cls=GuardOp, wrapfn="newint"),
+    hash = interp_attrproperty("hash", cls=GuardOp, wrapfn="newint"),
     )
 GuardOp.typedef.acceptable_as_base_class = False
 
@@ -222,9 +226,9 @@ DebugMergePoint.typedef = TypeDef(
     bytecode_no = GetSetProperty(DebugMergePoint.get_bytecode_no,
                                  doc="offset in the bytecode"),
     call_depth = interp_attrproperty("call_depth", cls=DebugMergePoint,
-                                     doc="Depth of calls within this loop"),
+                                     doc="Depth of calls within this loop", wrapfn="newint"),
     call_id = interp_attrproperty("call_id", cls=DebugMergePoint,
-                     doc="Number of applevel function traced in this loop"),
+                     doc="Number of applevel function traced in this loop", wrapfn="newint"),
     jitdriver_name = GetSetProperty(DebugMergePoint.get_jitdriver_name,
                      doc="Name of the jitdriver 'pypyjit' in the case "
                                     "of the main interpreter loop"),
@@ -276,18 +280,18 @@ class W_JitLoopInfo(W_Root):
         if self.type == "bridge":
             code_repr = 'bridge no %d' % self.bridge_no
         else:
-            code_repr = space.str_w(space.repr(self.w_green_key))
-        return space.wrap('<JitLoopInfo %s, %d operations, starting at <%s>>' %
-                          (self.jd_name, lgt, code_repr))
+            code_repr = space.text_w(space.repr(self.w_green_key))
+        return space.newtext('<JitLoopInfo %s, %d operations, starting at <%s>>' %
+                             (self.jd_name, lgt, code_repr))
 
     def descr_get_bridge_no(self, space):
         if space.is_none(self.w_green_key):
-            return space.wrap(self.bridge_no)
-        raise OperationError(space.w_TypeError, space.wrap("not a bridge"))
+            return space.newint(self.bridge_no)
+        raise oefmt(space.w_TypeError, "not a bridge")
 
 
 @unwrap_spec(loopno=int, asmaddr=int, asmlen=int, loop_no=int,
-             type=str, jd_name=str, bridge_no=int)
+             type='text', jd_name='text', bridge_no=int)
 def descr_new_jit_loop_info(space, w_subtype, w_greenkey, w_ops, loopno,
                             asmaddr, asmlen, loop_no, type, jd_name,
                             bridge_no=-1):
@@ -307,7 +311,8 @@ W_JitLoopInfo.typedef = TypeDef(
     __doc__ = W_JitLoopInfo.__doc__,
     __new__ = interp2app(descr_new_jit_loop_info),
     jitdriver_name = interp_attrproperty('jd_name', cls=W_JitLoopInfo,
-                       doc="Name of the JitDriver, pypyjit for the main one"),
+                       doc="Name of the JitDriver, pypyjit for the main one",
+                       wrapfn="newtext"),
     greenkey = interp_attrproperty_w('w_green_key', cls=W_JitLoopInfo,
                doc="Representation of place where the loop was compiled. "
                     "In the case of the main interpreter loop, it's a triplet "
@@ -315,15 +320,19 @@ W_JitLoopInfo.typedef = TypeDef(
     operations = interp_attrproperty_w('w_ops', cls=W_JitLoopInfo, doc=
                                        "List of operations in this loop."),
     loop_no = interp_attrproperty('loop_no', cls=W_JitLoopInfo, doc=
-                                  "Loop cardinal number"),
+                                  "Loop cardinal number",
+                                  wrapfn="newint"),
     bridge_no = GetSetProperty(W_JitLoopInfo.descr_get_bridge_no,
                                doc="bridge number (if a bridge)"),
     type = interp_attrproperty('type', cls=W_JitLoopInfo,
-                               doc="Loop type"),
+                               doc="Loop type",
+                               wrapfn="newtext"),
     asmaddr = interp_attrproperty('asmaddr', cls=W_JitLoopInfo,
-                                  doc="Address of machine code"),
+                                  doc="Address of machine code",
+                                  wrapfn="newint"),
     asmlen = interp_attrproperty('asmlen', cls=W_JitLoopInfo,
-                                  doc="Length of machine code"),
+                                  doc="Length of machine code",
+                                  wrapfn="newint"),
     __repr__ = interp2app(W_JitLoopInfo.descr_repr),
 )
 W_JitLoopInfo.typedef.acceptable_as_base_class = False
@@ -357,28 +366,27 @@ def get_stats_snapshot(space):
     w_times = space.newdict()
     if ll_times:
         for i in range(len(ll_times)):
-            w_key = space.newtuple([space.wrap(ll_times[i].type),
-                                    space.wrap(ll_times[i].number)])
+            w_key = space.newtuple([space.newtext(ll_times[i].type),
+                                    space.newint(ll_times[i].number)])
             space.setitem(w_times, w_key,
-                          space.wrap(ll_times[i].counter))
+                          space.newint(ll_times[i].counter))
     w_counters = space.newdict()
     for i, counter_name in enumerate(Counters.counter_names):
         v = jit_hooks.stats_get_counter_value(None, i)
-        space.setitem_str(w_counters, counter_name, space.wrap(v))
+        space.setitem_str(w_counters, counter_name, space.newint(v))
     w_counter_times = space.newdict()
     tr_time = jit_hooks.stats_get_times_value(None, Counters.TRACING)
-    space.setitem_str(w_counter_times, 'TRACING', space.wrap(tr_time))
+    space.setitem_str(w_counter_times, 'TRACING', space.newfloat(tr_time))
     b_time = jit_hooks.stats_get_times_value(None, Counters.BACKEND)
-    space.setitem_str(w_counter_times, 'BACKEND', space.wrap(b_time))
-    return space.wrap(W_JitInfoSnapshot(space, w_times, w_counters,
-                                        w_counter_times))
+    space.setitem_str(w_counter_times, 'BACKEND', space.newfloat(b_time))
+    return W_JitInfoSnapshot(space, w_times, w_counters, w_counter_times)
 
 def get_stats_asmmemmgr(space):
     """Returns the raw memory currently used by the JIT backend,
     as a pair (total_memory_allocated, memory_in_use)."""
     m1 = jit_hooks.stats_asmmemmgr_allocated(None)
     m2 = jit_hooks.stats_asmmemmgr_used(None)
-    return space.newtuple([space.wrap(m1), space.wrap(m2)])
+    return space.newtuple([space.newint(m1), space.newint(m2)])
 
 def enable_debug(space):
     """ Set the jit debugging - completely necessary for some stats to work,
