@@ -80,7 +80,6 @@ class TestCall(BaseTestPyPyC):
         assert log.opnames(ops) == []
         #
         assert entry_bridge.match_by_id('call', """
-            dummy_get_utf8?
             p38 = call_r(ConstClass(_ll_1_threadlocalref_get__Ptr_GcStruct_objectLlT_Signed), #, descr=<Callr . i EF=1 OS=5>)
             p39 = getfield_gc_r(p38, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_topframeref .*>)
             i40 = force_token()
@@ -115,8 +114,6 @@ class TestCall(BaseTestPyPyC):
                     self.a = a
                 def f(self, i):
                     return self.a + i
-            a = A("a") # stop field unboxing
-
             i = 0
             a = A(1)
             while i < n:
@@ -389,9 +386,10 @@ class TestCall(BaseTestPyPyC):
             {{{
             setfield_gc(p0, p20, descr=<FieldP .*PyFrame.vable_token .*>)
             setfield_gc(p22, ConstPtr(null), descr=<FieldP pypy.interpreter.argument.Arguments.inst_keywords_w .*>)
-            setfield_gc(p22, ConstPtr(null), descr=<FieldP pypy.interpreter.argument.Arguments.inst_keyword_names_w .*>)
+            setfield_gc(p22, ConstPtr(null), descr=<FieldP pypy.interpreter.argument.Arguments.inst_keywords .*>)
             setfield_gc(p22, 1, descr=<FieldU pypy.interpreter.argument.Arguments.inst__jit_few_keywords .*>)
             setfield_gc(p22, 0, descr=<FieldU pypy.interpreter.argument.Arguments.inst_methodcall .*>)
+            setfield_gc(p22, ConstPtr(null), descr=<FieldP pypy.interpreter.argument.Arguments.inst_keyword_names_w .*>)
             setfield_gc(p26, ConstPtr(ptr22), descr=<FieldP pypy.objspace.std.listobject.W_ListObject.inst_strategy .*>)
             setfield_gc(p26, ConstPtr(null), descr=<FieldP pypy.objspace.std.listobject.W_ListObject.inst_lstorage .*>)
             setarrayitem_gc(p24, 0, p26, descr=<ArrayP .>)
@@ -405,7 +403,7 @@ class TestCall(BaseTestPyPyC):
         def main(n):
             i = 1
             while i < n:
-                i += len(range(i+1)) - i
+                i += len(xrange(i+1)) - i
             return i
 
         log = self.run(main, [10000])
@@ -441,7 +439,6 @@ class TestCall(BaseTestPyPyC):
             i22 = getfield_gc_i(p12, descr=<FieldS pypy.objspace.std.intobject.W_IntObject.inst_intval .*>)
             i24 = int_lt(i22, 5000)
             guard_true(i24, descr=...)
-            dummy_get_utf8?
             guard_not_invalidated(descr=...)
             p29 = call_r(ConstClass(_ll_1_threadlocalref_get__Ptr_GcStruct_objectLlT_Signed), #, descr=<Callr . i EF=1 OS=5>)
             p30 = getfield_gc_r(p29, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_topframeref .*>)
@@ -451,7 +448,11 @@ class TestCall(BaseTestPyPyC):
             i34 = getfield_gc_i(p29, descr=<FieldU pypy.interpreter.executioncontext.ExecutionContext.inst_profilefunc .*>)
             i35 = int_is_zero(i34)
             guard_true(i35, descr=...)
-            i40 = int_add(i22, 5)
+            p37 = getfield_gc_r(ConstPtr(ptr36), descr=<FieldP pypy.interpreter.nestedscope.Cell.inst_w_value .*>)
+            guard_nonnull_class(p37, ConstClass(W_IntObject), descr=...)
+            i39 = getfield_gc_i(p37, descr=<FieldS pypy.objspace.std.intobject.W_IntObject.inst_intval .*>)
+            i40 = int_add_ovf(i22, i39)
+            guard_no_overflow(descr=...)
             --TICK--
         """)
 
@@ -555,26 +556,6 @@ class TestCall(BaseTestPyPyC):
         assert len(calls) == 0
         assert len([op for op in allops if op.name.startswith('new')]) == 0
 
-    def test_kwargs_update_virtual1(self):
-        log = self.run("""
-        def f(**kwargs):
-            return len(kwargs)
-        def main(stop):
-            i = 0
-            res = 0
-            while i < stop:
-                d = {'a': 1, 'b': 2}
-                # used to force the dict!
-                res += f(c=2, **d) # ID: call
-                i += 1
-            return res
-        """, [1000])
-        loop, = log.loops_by_id('call')
-        ops = loop.ops_by_id('call')
-        assert log.opnames(ops) == ["guard_not_invalidated", "force_token",
-                "int_add_ovf", "guard_no_overflow"]
-
-
     def test_kwargs_non_virtual(self):
         log = self.run("""
         def f(a, b, c):
@@ -594,7 +575,7 @@ class TestCall(BaseTestPyPyC):
         calls = [op for op in allops if op.name.startswith('call')]
         assert OpMatcher(calls).match('''
         p93 = call_r(ConstClass(view_as_kwargs), p35, p12, descr=<.*>)
-        i103 = call_i(ConstClass(_match_keywords), ConstPtr(ptr52), 0, 0, 0, p94, p98, 0, descr=<.*>)
+        i103 = call_i(ConstClass(_match_keywords), ConstPtr(ptr52), 0, 0, p94, p98, 0, descr=<.*>)
         ''')
         assert len([op for op in allops if op.name.startswith('new')]) == 1
         # 1 alloc
@@ -649,42 +630,3 @@ class TestCall(BaseTestPyPyC):
                 i += 1
             return 13
         """, [1000])
-
-
-    def test_nonstd_jitdriver_distinguishes_map(self):
-        log = self.run("""
-        def f(a):
-            return a + 1
-        def g(a):
-            return a + 2
-        def main():
-            # test the "contains" jitdriver, but the others are the same
-            res = (9999 in map(f, range(100000)))
-            res += (9999 in map(g, range(200000)))
-            return res
-        """, [])
-        assert len([l for l in log.loops if l.chunks[1].bytecode_name.startswith("DescrOperation.contains")]) == 2
-
-    def test_methodcall_kwargs_regression(self):
-        log = self.run("""
-        class A:
-            def f(self, x, y, z):
-                return x + y + z
-        def main():
-            a = A()
-            res = 0
-            for i in range(10000):
-                a.f(x=i, y=i+1, z=i*2) # ID: meth
-                res += i
-        """, [])
-        
-        loop, = log.loops_by_id('meth')
-
-        assert loop.match_by_id("meth", """
-            setfield_gc(p15, i65, descr=...)
-            guard_not_invalidated(descr=...)
-            i68 = int_mul_ovf(i62, 2)
-            guard_no_overflow(descr=...)
-            p69 = force_token()
-        """)
-
