@@ -1,27 +1,20 @@
-import os, pytest, sys
+import os, py, sys
 import signal as cpy_signal
-from pypy.tool.pytest.objspace import gettestobjspace
 
-GET_POSIX = "(): import %s as m ; return m" % os.name
-USEMODULES = ['posix', 'signal']
-
-def setup_module(mod):
-    mod.space = gettestobjspace(usemodules=USEMODULES)
 
 class TestCheckSignals:
-    spaceconfig = {'usemodules': USEMODULES}
+    spaceconfig = dict(usemodules=['signal'])
 
     def setup_class(cls):
         if not hasattr(os, 'kill') or not hasattr(os, 'getpid'):
-            pytest.skip("requires os.kill() and os.getpid()")
+            py.test.skip("requires os.kill() and os.getpid()")
         if not hasattr(cpy_signal, 'SIGUSR1'):
-            pytest.skip("requires SIGUSR1 in signal")
+            py.test.skip("requires SIGUSR1 in signal")
 
     def test_checksignals(self):
-        import os
         space = self.space
         w_received = space.appexec([], """():
-            import _signal as signal
+            import signal
             received = []
             def myhandler(signum, frame):
                 received.append(signum)
@@ -37,35 +30,32 @@ class TestCheckSignals:
         assert not space.is_true(w_received)
         #
         # calling ec.checksignals() should call it
-        print(space.getexecutioncontext().checksignals)
         space.getexecutioncontext().checksignals()
         assert space.is_true(w_received)
 
 
 class AppTestSignal:
     spaceconfig = {
-        "usemodules": ['signal', 'time', '_socket'] + (['fcntl'] if os.name != 'nt' else []),
+        "usemodules": ['signal', 'time'] + (['fcntl'] if os.name != 'nt' else []),
     }
 
     def setup_class(cls):
+        cls.w_signal = cls.space.getbuiltinmodule('signal')
         cls.w_temppath = cls.space.wrap(
-            str(pytest.ensuretemp("signal").join("foo.txt")))
-        cls.w_appdirect = cls.space.wrap(cls.runappdirect)
-        cls.w_posix = space.appexec([], GET_POSIX)
+            str(py.test.ensuretemp("signal").join("foo.txt")))
 
     def test_exported_names(self):
-        import sys, _signal
-        _signal.__dict__   # crashes if the interpleveldefs are invalid
-        if sys.platform == 'win32':
-            assert _signal.CTRL_BREAK_EVENT == 1
-            assert _signal.CTRL_C_EVENT == 0
+        import os
+        self.signal.__dict__   # crashes if the interpleveldefs are invalid
+        if os.name == 'nt':
+            assert self.signal.CTRL_BREAK_EVENT == 1
+            assert self.signal.CTRL_C_EVENT == 0
 
     def test_basics(self):
-        import types, _signal
-        os = self.posix
+        import types, os
         if not hasattr(os, 'kill') or not hasattr(os, 'getpid'):
             skip("requires os.kill() and os.getpid()")
-        signal = _signal   # the signal module to test
+        signal = self.signal   # the signal module to test
         if not hasattr(signal, 'SIGUSR1'):
             skip("requires SIGUSR1 in signal")
         signum = signal.SIGUSR1
@@ -101,7 +91,7 @@ class AppTestSignal:
         """
         Test that signal.signal returns SIG_DFL if that is the current handler.
         """
-        from _signal import signal, SIGINT, SIG_DFL, SIG_IGN
+        from signal import signal, SIGINT, SIG_DFL, SIG_IGN
 
         try:
             for handler in SIG_DFL, SIG_IGN, lambda *a: None:
@@ -114,7 +104,7 @@ class AppTestSignal:
         """
         Test that signal.signal returns SIG_IGN if that is the current handler.
         """
-        from _signal import signal, SIGINT, SIG_DFL, SIG_IGN
+        from signal import signal, SIGINT, SIG_DFL, SIG_IGN
 
         try:
             for handler in SIG_DFL, SIG_IGN, lambda *a: None:
@@ -128,7 +118,7 @@ class AppTestSignal:
         Test that signal.signal returns a Python object if one is the current
         handler.
         """
-        from _signal import signal, SIGINT, SIG_DFL, SIG_IGN
+        from signal import signal, SIGINT, SIG_DFL, SIG_IGN
         def installed(*a):
             pass
 
@@ -143,14 +133,13 @@ class AppTestSignal:
         """
         Test that signal.getsignal returns the currently installed handler.
         """
-        from _signal import getsignal, signal, SIGINT, SIG_DFL, SIG_IGN
+        from signal import getsignal, signal, SIGINT, SIG_DFL, SIG_IGN
 
         def handler(*a):
             pass
 
         try:
-            if not self.appdirect:
-                assert getsignal(SIGINT) == SIG_DFL
+            assert getsignal(SIGINT) == SIG_DFL
             signal(SIGINT, SIG_DFL)
             assert getsignal(SIGINT) == SIG_DFL
             signal(SIGINT, SIG_IGN)
@@ -162,7 +151,7 @@ class AppTestSignal:
 
     def test_check_signum(self):
         import sys
-        from _signal import getsignal, signal, NSIG
+        from signal import getsignal, signal, NSIG
 
         # signum out of range fails
         raises(ValueError, getsignal, NSIG)
@@ -175,7 +164,7 @@ class AppTestSignal:
 
     def test_alarm(self):
         try:
-            from _signal import alarm, signal, SIG_DFL, SIGALRM
+            from signal import alarm, signal, SIG_DFL, SIGALRM
         except:
             skip('no alarm on this platform')
         import time
@@ -195,7 +184,7 @@ class AppTestSignal:
 
     def test_set_wakeup_fd(self):
         try:
-            import _signal as signal, posix, fcntl
+            import signal, posix, fcntl
         except ImportError:
             skip('cannot import posix or fcntl')
         def myhandler(signum, frame):
@@ -208,7 +197,7 @@ class AppTestSignal:
             except OSError:
                 pass
             else:
-                raise AssertionError("posix.read(fd_read, 1) succeeded?")
+                raise AssertionError("os.read(fd_read, 1) succeeded?")
         #
         fd_read, fd_write = posix.pipe()
         flags = fcntl.fcntl(fd_write, fcntl.F_GETFL, 0)
@@ -218,45 +207,26 @@ class AppTestSignal:
         flags = flags | posix.O_NONBLOCK
         fcntl.fcntl(fd_read, fcntl.F_SETFL, flags)
         #
-        old_wakeup = signal.set_wakeup_fd(fd_write, warn_on_full_buffer=False)
+        old_wakeup = signal.set_wakeup_fd(fd_write)
         try:
             cannot_read()
             posix.kill(posix.getpid(), signal.SIGINT)
             res = posix.read(fd_read, 1)
-            assert res == bytes([signal.SIGINT])
+            assert res == '\x00'
             cannot_read()
         finally:
             old_wakeup = signal.set_wakeup_fd(old_wakeup)
         #
         signal.signal(signal.SIGINT, signal.SIG_DFL)
 
-    def test_set_wakeup_fd_socket_result(self):
-        import _socket as socket
-        import _signal as signal
-        sock1 = socket.socket()
-        sock2 = socket.socket()
-        try:
-            sock1.setblocking(False)
-            fd1 = sock1.fileno()
-            sock2.setblocking(False)
-            fd2 = sock2.fileno()
-
-            signal.set_wakeup_fd(fd1)
-            signal.set_wakeup_fd(fd2) == fd1
-            assert signal.set_wakeup_fd(-1) == fd2
-        finally:
-            sock1.close()
-            sock2.close()
-
     def test_set_wakeup_fd_invalid(self):
-        import _signal as signal
+        import signal
         with open(self.temppath, 'wb') as f:
             fd = f.fileno()
-        raises((ValueError, OSError), signal.set_wakeup_fd, fd)
+        raises(ValueError, signal.set_wakeup_fd, fd)
 
     def test_siginterrupt(self):
-        import _signal as signal, time
-        os = self.posix
+        import signal, os, time
         if not hasattr(signal, 'siginterrupt'):
             skip('non siginterrupt in signal')
         signum = signal.SIGUSR1
@@ -290,39 +260,12 @@ class AppTestSignal:
         finally:
             signal.signal(signum, oldhandler)
 
-    def test_default_int_handler(self):
-        import signal
-        for args in [(), (1, 2)]:
-            try:
-                signal.default_int_handler(*args)
-            except KeyboardInterrupt:
-                pass
-            else:
-                raise AssertionError("did not raise!")
-
-    def test_valid_signals(self):
-        import signal, sys
-        s = signal.valid_signals()
-        assert isinstance(s, set)
-        assert signal.Signals.SIGINT in s
-        if sys.platform != "win32":
-            assert signal.Signals.SIGALRM in s
-        assert 0 not in s
-        assert signal.NSIG not in s
-        assert len(s) < signal.NSIG
-
-    def test_strsignal(self):
-        import signal
-        assert signal.strsignal(signal.Signals.SIGSEGV) == "Segmentation fault"
-        raises(ValueError, signal.strsignal, 4242)
-
-
 class AppTestSignalSocket:
     spaceconfig = dict(usemodules=['signal', '_socket'])
 
     def test_alarm_raise(self):
         try:
-            from _signal import alarm, signal, SIG_DFL, SIGALRM
+            from signal import alarm, signal, SIG_DFL, SIGALRM
         except ImportError:
             skip("no SIGALRM on this platform")
         import _socket
@@ -337,7 +280,7 @@ class AppTestSignalSocket:
             signal(SIGALRM, handler)
             alarm(1)
             try:
-                s._accept()
+                s.accept()
             except Alarm:
                 pass
             else:
@@ -346,13 +289,15 @@ class AppTestSignalSocket:
         finally:
             signal(SIGALRM, SIG_DFL)
 
-
-@pytest.mark.skipif(sys.platform == 'win32', reason='posix-only')
 class AppTestItimer:
     spaceconfig = dict(usemodules=['signal'])
 
+    def setup_class(cls):
+        if sys.platform == 'win32':
+            py.test.skip("Unix only")
+
     def test_itimer_real(self):
-        import _signal as signal
+        import signal
 
         def sig_alrm(*args):
             self.called = True
@@ -369,84 +314,6 @@ class AppTestItimer:
         assert self.called
 
     def test_itimer_exc(self):
-        import _signal as signal
+        import signal
 
         raises(signal.ItimerError, signal.setitimer, -1, 0)
-
-@pytest.mark.skipif(sys.platform == 'win32', reason='posix-only')
-class AppTestPThread:
-    spaceconfig = dict(usemodules=['signal', 'thread', 'time'])
-
-    def test_pthread_kill(self):
-        import _signal as signal
-        import _thread
-        signum = signal.SIGUSR1
-        def handler(signum, frame):
-            1/0
-        signal.signal(signum, handler)
-        tid = _thread.get_ident()
-        raises(ZeroDivisionError, signal.pthread_kill, tid, signum)
-        
-    def test_sigwait(self):
-        import _signal as signal
-        def handler(signum, frame):
-            1/0
-        signal.signal(signal.SIGALRM, handler)
-        signal.alarm(1)
-        received = signal.sigwait([signal.SIGALRM])
-        assert received == signal.SIGALRM
-        
-    def test_sigmask(self):
-        import _signal as signal, posix
-        signum1 = signal.SIGUSR1
-        signum2 = signal.SIGUSR2
-
-        def handler(signum, frame):
-            pass
-        signal.signal(signum1, handler)
-        signal.signal(signum2, handler)
-
-        signal.pthread_sigmask(signal.SIG_BLOCK, (signum1, signum2))
-        posix.kill(posix.getpid(), signum1)
-        posix.kill(posix.getpid(), signum2)
-        assert signal.sigpending() == set((signum1, signum2))
-        # Unblocking the 2 signals calls the C signal handler twice
-        signal.pthread_sigmask(signal.SIG_UNBLOCK, (signum1, signum2))
-        assert signal.sigpending() == set()
-
-    def test_raise_signal(self):
-        import types, _signal, sys
-        signal = _signal   # the signal module to test
-        if sys.platform == 'win32':
-            raises(OSError, signal.raise_signal, 1)
-        if not hasattr(signal, 'SIGUSR1'):
-            skip("requires SIGUSR1 in signal")
-        signum = signal.SIGUSR1
-
-        received = []
-        def myhandler(signum, frame):
-            assert isinstance(frame, types.FrameType)
-            received.append(signum)
-        signal.signal(signum, myhandler)
-
-        signal.raise_signal(signum)
-        # the signal should be delivered to the handler immediately
-        assert received == [signum]
-        del received[:]
-
-        signal.raise_signal(signum)
-        # the signal should be delivered to the handler immediately
-        assert received == [signum]
-        del received[:]
-
-        signal.signal(signum, signal.SIG_IGN)
-
-        signal.raise_signal(signum)
-        for i in range(10000):
-            # wait a bit - signal should not arrive
-            if received:
-                break
-        assert received == []
-
-        signal.signal(signum, signal.SIG_DFL)
-
