@@ -5,14 +5,14 @@ from rpython.rlib.objectmodel import we_are_translated
 def getdefaultencoding(space):
     """Return the current default string encoding used by the Unicode
 implementation."""
-    return space.wrap(space.sys.defaultencoding)
+    return space.newtext(space.sys.defaultencoding)
 
 def setdefaultencoding(space, w_encoding):
     """Set the current default string encoding used by the Unicode
 implementation."""
-    encoding = space.str_w(w_encoding)
+    encoding = space.text_w(w_encoding)
     mod = space.getbuiltinmodule("_codecs")
-    w_lookup = space.getattr(mod, space.wrap("lookup"))
+    w_lookup = space.getattr(mod, space.newtext("lookup"))
     # check whether the encoding is there
     space.call_function(w_lookup, w_encoding)
     space.sys.w_default_encoder = None
@@ -21,11 +21,11 @@ implementation."""
 def get_w_default_encoder(space):
     assert not (space.config.translating and not we_are_translated()), \
         "get_w_default_encoder() should not be called during translation"
-    w_encoding = space.wrap(space.sys.defaultencoding)
+    w_encoding = space.newtext(space.sys.defaultencoding)
     mod = space.getbuiltinmodule("_codecs")
-    w_lookup = space.getattr(mod, space.wrap("lookup"))
+    w_lookup = space.getattr(mod, space.newtext("lookup"))
     w_functuple = space.call_function(w_lookup, w_encoding)
-    w_encoder = space.getitem(w_functuple, space.wrap(0))
+    w_encoder = space.getitem(w_functuple, space.newint(0))
     space.sys.w_default_encoder = w_encoder    # cache it
     return w_encoder
 
@@ -51,7 +51,7 @@ def _getfilesystemencoding(space):
                 if loc_codeset:
                     codecmod = space.getbuiltinmodule('_codecs')
                     w_res = space.call_method(codecmod, 'lookup',
-                                              space.wrap(loc_codeset))
+                                              space.newtext(loc_codeset))
                     if space.is_true(w_res):
                         encoding = loc_codeset
             finally:
@@ -66,4 +66,4 @@ def getfilesystemencoding(space):
     """
     if space.sys.filesystemencoding is None:
         space.sys.filesystemencoding = _getfilesystemencoding(space)
-    return space.wrap(space.sys.filesystemencoding)
+    return space.newtext(space.sys.filesystemencoding)
