@@ -6,17 +6,21 @@ indirection is introduced to make the version tag change less often.
 from rpython.rlib import jit, rerased, objectmodel
 
 from pypy.interpreter.baseobjspace import W_Root
-from pypy.interpreter.error import OperationError
 from pypy.objspace.std.dictmultiobject import (
     DictStrategy, ObjectDictStrategy, _never_equal_to_string,
     create_iterator_classes)
 from pypy.objspace.std.typeobject import (
-    MutableCell, IntMutableCell, ObjectMutableCell, write_cell, unwrap_cell)
+    MutableCell, IntMutableCell, ObjectMutableCell, write_cell)
 
 
 class VersionTag(object):
     pass
 
+def unwrap_cell(space, w_value):
+    if isinstance(w_value, MutableCell):
+        return w_value.unwrap_cell(space)
+    return w_value
+
 
 def _wrapkey(space, key):
     return space.newtext(key)
@@ -50,21 +54,10 @@ class ModuleDictStrategy(DictStrategy):
     def _getdictvalue_no_unwrapping_pure(self, version, w_dict, key):
         return self.unerase(w_dict.dstorage).get(key, None)
 
-    def try_unwrap_key(self, space, w_key):
-        if space.is_w(space.type(w_key), space.w_text):
-            try:
-                return space.text_w(w_key)
-            except OperationError as e:
-                if e.match(space, space.w_UnicodeEncodeError):
-                    return None
-                raise
-        return None
-
     def setitem(self, w_dict, w_key, w_value):
         space = self.space
-        key = self.try_unwrap_key(space, w_key)
-        if key is not None:
-            self.setitem_str(w_dict, key, w_value)
+        if space.is_w(space.type(w_key), space.w_text):
+            self.setitem_str(w_dict, space.text_w(w_key), w_value)
         else:
             self.switch_to_object_strategy(w_dict)
             w_dict.setitem(w_key, w_value)
@@ -82,8 +75,8 @@ class ModuleDictStrategy(DictStrategy):
 
     def setdefault(self, w_dict, w_key, w_default):
         space = self.space
-        key = self.try_unwrap_key(space, w_key)
-        if key is not None:
+        if space.is_w(space.type(w_key), space.w_text):
+            key = space.text_w(w_key)
             cell = self.getdictvalue_no_unwrapping(w_dict, key)
             w_result = unwrap_cell(self.space, cell)
             if w_result is not None:
@@ -97,8 +90,8 @@ class ModuleDictStrategy(DictStrategy):
     def delitem(self, w_dict, w_key):
         space = self.space
         w_key_type = space.type(w_key)
-        key = self.try_unwrap_key(space, w_key)
-        if key is not None:
+        if space.is_w(w_key_type, space.w_text):
+            key = space.text_w(w_key)
             dict_w = self.unerase(w_dict.dstorage)
             try:
                 del dict_w[key]
@@ -118,9 +111,9 @@ class ModuleDictStrategy(DictStrategy):
     def getitem(self, w_dict, w_key):
         space = self.space
         w_lookup_type = space.type(w_key)
-        key = self.try_unwrap_key(space, w_key)
-        if key is not None:
-            return self.getitem_str(w_dict, key)
+        if space.is_w(w_lookup_type, space.w_text):
+            return self.getitem_str(w_dict, space.text_w(w_key))
+
         elif _never_equal_to_string(space, w_lookup_type):
             return None
         else:
@@ -176,24 +169,10 @@ class ModuleDictStrategy(DictStrategy):
     def getiteritems_with_hash(self, w_dict):
         return objectmodel.iteritems_with_hash(self.unerase(w_dict.dstorage))
 
-    def getiterreversed(self, w_dict):
-        return objectmodel.reversed_dict(self.unerase(w_dict.dstorage))
-
     wrapkey = _wrapkey
 
     def wrapvalue(space, value):
         return unwrap_cell(space, value)
 
-def remove_cell(w_dict, space, name):
-    from pypy.objspace.std.dictmultiobject import W_DictMultiObject
-    if isinstance(w_dict, W_DictMultiObject):
-        strategy = w_dict.get_strategy()
-        if isinstance(strategy, ModuleDictStrategy):
-            w_value = strategy.getitem_str(w_dict, name)
-            dict_w = strategy.unerase(w_dict.dstorage)
-            strategy.mutated()
-            dict_w[name] = w_value # store without cell
-    # nothing to do, not a W_DictMultiObject or wrong strategy
-
 
 create_iterator_classes(ModuleDictStrategy)
