@@ -172,7 +172,7 @@ def _truediv(space, x, y):
     # floating-point division
     a = float(x)
     b = float(y)
-    return space.wrap(a / b)
+    return space.newfloat(a / b)
 
 
 def _mod(space, x, y):
@@ -190,8 +190,7 @@ def _divmod(space, x, y):
         raise oefmt(space.w_ZeroDivisionError, "integer divmod by zero")
     # no overflow possible
     m = x % y
-    w = space.wrap
-    return space.newtuple([w(z), w(m)])
+    return space.newtuple([space.newint(z), space.newint(m)])
 
 
 def _divmod_ovf2small(space, x, y):
@@ -423,10 +422,10 @@ class W_IntObject(W_AbstractIntObject):
         return space.newfloat(x)
 
     def descr_oct(self, space):
-        return space.wrap(oct(self.intval))
+        return space.newtext(oct(self.intval))
 
     def descr_hex(self, space):
-        return space.wrap(hex(self.intval))
+        return space.newtext(hex(self.intval))
 
     def descr_getnewargs(self, space):
         return space.newtuple([wrapint(space, self.intval)])
@@ -441,11 +440,11 @@ class W_IntObject(W_AbstractIntObject):
         while val:
             bits += 1
             val >>= 1
-        return space.wrap(bits)
+        return space.newint(bits)
 
     def descr_repr(self, space):
         res = str(self.intval)
-        return space.wrap(res)
+        return space.newtext(res)
     descr_str = func_with_new_name(descr_repr, 'descr_str')
 
     def descr_format(self, space, w_format_spec):
@@ -478,7 +477,7 @@ class W_IntObject(W_AbstractIntObject):
             result = _pow(space, x, y, z)
         except (OverflowError, ValueError):
             return _pow_ovf2long(space, x, y, w_modulus)
-        return space.wrap(result)
+        return space.newint(result)
 
     @unwrap_spec(w_modulus=WrappedDefault(None))
     def descr_rpow(self, space, w_base, w_modulus=None):
@@ -626,7 +625,7 @@ def wrapint(space, x):
     # use r_uint to perform a single comparison (this whole function is
     # getting inlined into every caller so keeping the branching to a
     # minimum is a good idea)
-    index = r_uint(x - lower)
+    index = r_uint(x) - r_uint(lower)
     if index >= r_uint(upper - lower):
         w_res = instantiate(W_IntObject)
     else:
@@ -695,9 +694,9 @@ def _new_int(space, w_inttype, w_x, w_base=None):
             # we cannot construct a subclass of int instance with an
             # an overflowing long
             value = space.int_w(w_obj, allow_conversion=False)
-        elif space.isinstance_w(w_value, space.w_str):
+        elif space.isinstance_w(w_value, space.w_bytes):
             value, w_longval = _string_to_int_or_long(space, w_value,
-                                                      space.str_w(w_value))
+                                                      space.text_w(w_value))
         elif space.isinstance_w(w_value, space.w_unicode):
             from pypy.objspace.std.unicodeobject import unicode_to_decimal_w
             string = unicode_to_decimal_w(space, w_value)
@@ -722,7 +721,7 @@ def _new_int(space, w_inttype, w_x, w_base=None):
             s = unicode_to_decimal_w(space, w_value)
         else:
             try:
-                s = space.str_w(w_value)
+                s = space.text_w(w_value)
             except OperationError as e:
                 raise oefmt(space.w_TypeError,
                             "int() can't convert non-string with explicit "
