@@ -1,8 +1,7 @@
 from pypy.interpreter.error import oefmt
-from pypy.objspace.std.tupleobject import (W_AbstractTupleObject,
-    XXPRIME_1, XXPRIME_2, XXPRIME_5, xxrotate, uhash_type)
+from pypy.objspace.std.tupleobject import W_AbstractTupleObject
 from pypy.objspace.std.util import negate
-from rpython.rlib.objectmodel import specialize
+from rpython.rlib.objectmodel import compute_hash, specialize
 from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.tool.sourcetools import func_with_new_name
@@ -65,32 +64,30 @@ def make_specialised_class(typetuple):
         getitems_copy = func_with_new_name(tolist, 'getitems_copy')
 
         def descr_hash(self, space):
-            acc = XXPRIME_5
-
+            mult = 1000003
+            x = 0x345678
+            z = typelen
             for i in iter_n:
                 value = getattr(self, 'value%s' % i)
                 if typetuple[i] == object:
-                    lane = uhash_type(space.int_w(space.hash(value)))
+                    y = space.int_w(space.hash(value))
+                elif typetuple[i] == int:
+                    # mimic cpythons behavior of a hash value of -2 for -1
+                    y = value
+                    y -= (y == -1)  # No explicit condition, to avoid JIT bridges
                 elif typetuple[i] == float:
                     # get the correct hash for float which is an
                     # integer & other less frequent cases
                     from pypy.objspace.std.floatobject import _hash_float
-                    lane = uhash_type(_hash_float(space, value))
-                elif typetuple[i] == int:
-                    # hash for int which is different from the hash
-                    # given by rpython
-                    from pypy.objspace.std.intobject import _hash_int
-                    lane = uhash_type(_hash_int(value))
+                    y = _hash_float(space, value)
+                    y -= (y == -1)
                 else:
-                    raise NotImplementedError
-
-                acc += lane * XXPRIME_2
-                acc = xxrotate(acc)
-                acc *= XXPRIME_1
-
-            acc += typelen ^ (XXPRIME_5 ^ uhash_type(3527539))
-            acc += (acc == uhash_type(-1)) * uhash_type(1546275796 + 1)
-            return space.newint(intmask(acc))
+                    assert 0, "unreachable"
+                x = (x ^ y) * mult
+                z -= 1
+                mult += 82520 + z + z
+            x += 97531
+            return space.newint(intmask(x))
 
         def descr_eq(self, space, w_other):
             if not isinstance(w_other, W_AbstractTupleObject):
@@ -147,14 +144,13 @@ Cls_oo = make_specialised_class((object, object))
 Cls_ff = make_specialised_class((float, float))
 
 def makespecialisedtuple(space, list_w):
-    from pypy.objspace.std.listobject import is_plain_int1, plain_int_w
+    from pypy.objspace.std.intobject import W_IntObject
     from pypy.objspace.std.floatobject import W_FloatObject
     if len(list_w) == 2:
         w_arg1, w_arg2 = list_w
-        if is_plain_int1(w_arg1):
-            if is_plain_int1(w_arg2):
-                return Cls_ii(space, plain_int_w(space, w_arg1),
-                                     plain_int_w(space, w_arg2))
+        if type(w_arg1) is W_IntObject:
+            if type(w_arg2) is W_IntObject:
+                return Cls_ii(space, space.int_w(w_arg1), space.int_w(w_arg2))
         elif type(w_arg1) is W_FloatObject:
             if type(w_arg2) is W_FloatObject:
                 return Cls_ff(space, space.float_w(w_arg1), space.float_w(w_arg2))
