@@ -2,7 +2,6 @@ import py
 from rpython.jit.metainterp.opencoder import Trace, untag, TAGINT, TAGBOX
 from rpython.jit.metainterp.resoperation import rop, AbstractResOp
 from rpython.jit.metainterp.history import ConstInt, IntFrontendOp
-from rpython.jit.metainterp.history import ConstPtrJitCode
 from rpython.jit.metainterp.optimizeopt.optimizer import Optimizer
 from rpython.jit.metainterp import resume
 from rpython.jit.metainterp.test.strategies import lists_of_operations
@@ -36,7 +35,7 @@ class FakeFrame(object):
         self.jitcode = jitcode
         self.boxes = boxes
 
-    def get_list_of_active_boxes(self, flag, new_array, encode, after_residual_call=False):
+    def get_list_of_active_boxes(self, flag, new_array, encode):
         a = new_array(len(self.boxes))
         for i, box in enumerate(self.boxes):
             a[i] = encode(box)
@@ -210,33 +209,5 @@ class TestOpencoder(object):
     def test_tag_overflow(self):
         t = Trace([], metainterp_sd)
         i0 = FakeOp(100000)
-        # if we overflow, we can keep recording
-        for i in range(10):
-            t.record_op(rop.FINISH, [i0])
-            assert t.unpack() == ([], [])
-        assert t.tag_overflow
-
-    def test_encode_caching(self):
-        from rpython.rtyper.lltypesystem import lltype, llmemory
-        S = lltype.GcStruct('S')
-        s1 = lltype.malloc(S)
-        t = Trace([], metainterp_sd)
-        c1 = ConstPtrJitCode(lltype.cast_opaque_ptr(llmemory.GCREF, s1))
-        i = t._cached_const_ptr(c1)
-        assert i == 1
-        assert c1.opencoder_index == 1
-
-        s2 = lltype.malloc(S)
-        c2 = ConstPtrJitCode(lltype.cast_opaque_ptr(llmemory.GCREF, s2))
-        i = t._cached_const_ptr(c2)
-        assert i == 2
-        assert c2.opencoder_index == 2
-
-        assert len(t._refs) == 3 # plus null ptr
-
-        # looking up again does not need the dict
-        t._bigints_dict = None
-        i = t._cached_const_ptr(c1)
-        assert i == 1
-        i = t._cached_const_ptr(c2)
-        assert i == 2
+        py.test.raises(SwitchToBlackhole, t.record_op, rop.FINISH, [i0])
+        assert t.unpack() == ([], [])
