@@ -1,8 +1,10 @@
 import py
 import inspect
 
+from rpython.rlib.objectmodel import compute_hash, compute_identity_hash
 from rpython.translator.c import gc
 from rpython.annotator import model as annmodel
+from rpython.rtyper.llannotation import SomePtr
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi, llgroup
 from rpython.memory.gctransform import framework, shadowstack
 from rpython.rtyper.lltypesystem.lloperation import llop, void
@@ -12,6 +14,8 @@ from rpython.rlib import rgc
 from rpython.conftest import option
 from rpython.rlib.rstring import StringBuilder
 from rpython.rlib.rarithmetic import LONG_BIT
+from rpython.rtyper.rtyper import llinterp_backend
+
 
 WORD = LONG_BIT // 8
 
@@ -26,9 +30,11 @@ def rtype(func, inputtypes, specialize=True, gcname='ref',
     t.config.set(**extraconfigopts)
     ann = t.buildannotator()
     ann.build_types(func, inputtypes)
+    rtyper = t.buildrtyper()
+    rtyper.backend = llinterp_backend
 
     if specialize:
-        t.buildrtyper().specialize()
+        rtyper.specialize()
     if backendopt:
         from rpython.translator.backendopt.all import backend_optimizations
         backend_optimizations(t)
@@ -41,7 +47,6 @@ ARGS = lltype.FixedSizeArray(lltype.Signed, 3)
 class GCTest(object):
     gcpolicy = None
     GC_CAN_MOVE = False
-    GC_CAN_MALLOC_NONMOVABLE = True
     taggedpointers = False
 
     def setup_class(cls):
@@ -98,7 +103,7 @@ class GCTest(object):
 
         from rpython.translator.c.genc import CStandaloneBuilder
 
-        s_args = annmodel.SomePtr(lltype.Ptr(ARGS))
+        s_args = SomePtr(lltype.Ptr(ARGS))
         t = rtype(entrypoint, [s_args], gcname=cls.gcname,
                   taggedpointers=cls.taggedpointers)
 
@@ -108,7 +113,8 @@ class GCTest(object):
 
         cbuild = CStandaloneBuilder(t, entrypoint, config=t.config,
                                     gcpolicy=cls.gcpolicy)
-        db = cbuild.generate_graphs_for_llinterp()
+        cbuild.make_entrypoint_wrapper = False
+        db = cbuild.build_database()
         entrypointptr = cbuild.getentrypointptr()
         entrygraph = entrypointptr._obj.graph
         if option.view:
@@ -153,7 +159,6 @@ class GCTest(object):
 
 class GenericGCTests(GCTest):
     GC_CAN_SHRINK_ARRAY = False
-
     def define_instances(cls):
         class A(object):
             pass
@@ -289,7 +294,7 @@ class GenericGCTests(GCTest):
         res = run([])
         assert res == 42
 
-    def define_finalizer(cls):
+    def define_destructor(cls):
         class B(object):
             pass
         b = B()
@@ -312,6 +317,68 @@ class GenericGCTests(GCTest):
             return b.num_deleted
         return f
 
+    def test_destructor(self):
+        run = self.runner("destructor")
+        res = run([5, 42]) #XXX pure lazyness here too
+        assert res == 6
+
+    def define_old_style_finalizer(cls):
+        class B(object):
+            pass
+        b = B()
+        b.nextid = 0
+        b.num_deleted = 0
+        class A(object):
+            def __init__(self):
+                self.id = b.nextid
+                b.nextid += 1
+            def __del__(self):
+                llop.gc__collect(lltype.Void)
+                b.num_deleted += 1
+        def f(x, y):
+            a = A()
+            i = 0
+            while i < x:
+                i += 1
+                a = A()
+            llop.gc__collect(lltype.Void)
+            llop.gc__collect(lltype.Void)
+            return b.num_deleted
+        return f
+
+    def test_old_style_finalizer(self):
+        run = self.runner("old_style_finalizer")
+        res = run([5, 42]) #XXX pure lazyness here too
+        assert res == 6
+
+    def define_finalizer(cls):
+        class B(object):
+            pass
+        b = B()
+        b.nextid = 0
+        b.num_deleted = 0
+        class A(object):
+            def __init__(self):
+                self.id = b.nextid
+                b.nextid += 1
+                fq.register_finalizer(self)
+        class FQ(rgc.FinalizerQueue):
+            Class = A
+            def finalizer_trigger(self):
+                while self.next_dead() is not None:
+                    b.num_deleted += 1
+        fq = FQ()
+        def f(x, y):
+            a = A()
+            i = 0
+            while i < x:
+                i += 1
+                a = A()
+            llop.gc__collect(lltype.Void)
+            llop.gc__collect(lltype.Void)
+            return b.num_deleted
+        return f
+
     def test_finalizer(self):
         run = self.runner("finalizer")
         res = run([5, 42]) #XXX pure lazyness here too
@@ -327,12 +394,20 @@ class GenericGCTests(GCTest):
             def __init__(self):
                 self.id = b.nextid
                 b.nextid += 1
-            def __del__(self):
-                b.num_deleted += 1
-                C()
+                fq.register_finalizer(self)
         class C(AAA):
-            def __del__(self):
-                b.num_deleted += 1
+            pass
+        class FQ(rgc.FinalizerQueue):
+            Class = AAA
+            def finalizer_trigger(self):
+                while True:
+                    a = self.next_dead()
+                    if a is None:
+                        break
+                    b.num_deleted += 1
+                    if not isinstance(a, C):
+                        C()
+        fq = FQ()
         def f(x, y):
             a = AAA()
             i = 0
@@ -359,9 +434,17 @@ class GenericGCTests(GCTest):
             def __init__(self):
                 self.id = b.nextid
                 b.nextid += 1
-            def __del__(self):
-                b.num_deleted += 1
-                b.a = self
+                fq.register_finalizer(self)
+        class FQ(rgc.FinalizerQueue):
+            Class = A
+            def finalizer_trigger(self):
+                while True:
+                    a = self.next_dead()
+                    if a is None:
+                        break
+                    b.num_deleted += 1
+                    b.a = a
+        fq = FQ()
         def f(x, y):
             a = A()
             i = 0
@@ -372,7 +455,7 @@ class GenericGCTests(GCTest):
             llop.gc__collect(lltype.Void)
             aid = b.a.id
             b.a = None
-            # check that __del__ is not called again
+            # check that finalizer_trigger() is not called again
             llop.gc__collect(lltype.Void)
             llop.gc__collect(lltype.Void)
             return b.num_deleted * 10 + aid + 100 * (b.a is None)
@@ -384,26 +467,20 @@ class GenericGCTests(GCTest):
         assert 160 <= res <= 165
 
     def define_custom_trace(cls):
-        from rpython.rtyper.annlowlevel import llhelper
-        from rpython.rtyper.lltypesystem import llmemory
         #
-        S = lltype.GcStruct('S', ('x', llmemory.Address), rtti=True)
+        S = lltype.GcStruct('S', ('x', llmemory.Address))
         T = lltype.GcStruct('T', ('z', lltype.Signed))
         offset_of_x = llmemory.offsetof(S, 'x')
-        def customtrace(obj, prev):
-            if not prev:
-                return obj + offset_of_x
-            else:
-                return llmemory.NULL
-        CUSTOMTRACEFUNC = lltype.FuncType([llmemory.Address, llmemory.Address],
-                                          llmemory.Address)
-        customtraceptr = llhelper(lltype.Ptr(CUSTOMTRACEFUNC), customtrace)
-        lltype.attachRuntimeTypeInfo(S, customtraceptr=customtraceptr)
+        def customtrace(gc, obj, callback, arg):
+            gc._trace_callback(callback, arg, obj + offset_of_x)
+        lambda_customtrace = lambda: customtrace
+
         #
         def setup():
-            s1 = lltype.malloc(S)
+            rgc.register_custom_trace_hook(S, lambda_customtrace)
             tx = lltype.malloc(T)
             tx.z = 4243
+            s1 = lltype.malloc(S)
             s1.x = llmemory.cast_ptr_to_adr(tx)
             return s1
         def f():
@@ -442,7 +519,7 @@ class GenericGCTests(GCTest):
         res = run([])
         assert res
 
-    def define_weakref_to_object_with_finalizer(cls):
+    def define_weakref_to_object_with_destructor(cls):
         import weakref, gc
         class A(object):
             count = 0
@@ -461,6 +538,36 @@ class GenericGCTests(GCTest):
             return result
         return f
 
+    def test_weakref_to_object_with_destructor(self):
+        run = self.runner("weakref_to_object_with_destructor")
+        res = run([])
+        assert res
+
+    def define_weakref_to_object_with_finalizer(cls):
+        import weakref, gc
+        class A(object):
+            count = 0
+        a = A()
+        class B(object):
+            pass
+        class FQ(rgc.FinalizerQueue):
+            Class = B
+            def finalizer_trigger(self):
+                while self.next_dead() is not None:
+                    a.count += 1
+        fq = FQ()
+        def g():
+            b = B()
+            fq.register_finalizer(b)
+            return weakref.ref(b)
+        def f():
+            ref = g()
+            llop.gc__collect(lltype.Void)
+            llop.gc__collect(lltype.Void)
+            result = a.count == 1 and (ref() is None)
+            return result
+        return f
+
     def test_weakref_to_object_with_finalizer(self):
         run = self.runner("weakref_to_object_with_finalizer")
         res = run([])
@@ -477,15 +584,24 @@ class GenericGCTests(GCTest):
             def __init__(self):
                 self.id = b.nextid
                 b.nextid += 1
-            def __del__(self):
-                llop.gc__collect(lltype.Void)
-                b.num_deleted += 1
-                C()
-                C()
+                fq.register_finalizer(self)
         class C(A):
-            def __del__(self):
-                b.num_deleted += 1
-                b.num_deleted_c += 1
+            pass
+        class FQ(rgc.FinalizerQueue):
+            Class = A
+            def finalizer_trigger(self):
+                while True:
+                    a = self.next_dead()
+                    if a is None:
+                        break
+                    llop.gc__collect(lltype.Void)
+                    b.num_deleted += 1
+                    if isinstance(a, C):
+                        b.num_deleted_c += 1
+                    else:
+                        C()
+                        C()
+        fq = FQ()
         def f(x, y):
             persistent_a1 = A()
             persistent_a2 = A()
@@ -620,45 +736,6 @@ class GenericGCTests(GCTest):
         res = run([])
         assert res == self.GC_CAN_MOVE
 
-    def define_malloc_nonmovable(cls):
-        TP = lltype.GcArray(lltype.Char)
-        def func():
-            #try:
-            a = rgc.malloc_nonmovable(TP, 3, zero=True)
-            rgc.collect()
-            if a:
-                assert not rgc.can_move(a)
-                return 1
-            return 0
-            #except Exception, e:
-            #    return 2
-
-        return func
-
-    def test_malloc_nonmovable(self):
-        run = self.runner("malloc_nonmovable")
-        assert int(self.GC_CAN_MALLOC_NONMOVABLE) == run([])
-
-    def define_malloc_nonmovable_fixsize(cls):
-        S = lltype.GcStruct('S', ('x', lltype.Float))
-        TP = lltype.GcStruct('T', ('s', lltype.Ptr(S)))
-        def func():
-            try:
-                a = rgc.malloc_nonmovable(TP)
-                rgc.collect()
-                if a:
-                    assert not rgc.can_move(a)
-                    return 1
-                return 0
-            except Exception, e:
-                return 2
-
-        return func
-
-    def test_malloc_nonmovable_fixsize(self):
-        run = self.runner("malloc_nonmovable_fixsize")
-        assert run([]) == int(self.GC_CAN_MALLOC_NONMOVABLE)
-
     def define_shrink_array(cls):
         from rpython.rtyper.lltypesystem.rstr import STR
 
@@ -706,9 +783,7 @@ class GenericGCTests(GCTest):
 
 class GenericMovingGCTests(GenericGCTests):
     GC_CAN_MOVE = True
-    GC_CAN_MALLOC_NONMOVABLE = False
     GC_CAN_TEST_ID = False
-
     def define_many_ids(cls):
         class A(object):
             pass
@@ -766,7 +841,7 @@ class GenericMovingGCTests(GenericGCTests):
         def g():
             r = lltype.malloc(P)
             r.x = 1
-            p = llop.do_malloc_fixedsize_clear(llmemory.GCREF)  # placeholder
+            p = llop.do_malloc_fixedsize(llmemory.GCREF)  # placeholder
             p = lltype.cast_opaque_ptr(lltype.Ptr(P), p)
             p.x = r.x
             return p.x
@@ -793,10 +868,10 @@ class GenericMovingGCTests(GenericGCTests):
 
             type_id = layoutbuilder.get_type_id(P)
             #
-            # now fix the do_malloc_fixedsize_clear in the graph of g
+            # now fix the do_malloc_fixedsize in the graph of g
             graph = graphof(translator, g)
             for op in graph.startblock.operations:
-                if op.opname == 'do_malloc_fixedsize_clear':
+                if op.opname == 'do_malloc_fixedsize':
                     op.args = [Constant(type_id, llgroup.HALFWORD),
                                Constant(llmemory.sizeof(P), lltype.Signed),
                                Constant(False, lltype.Bool), # has_finalizer
@@ -814,7 +889,7 @@ class GenericMovingGCTests(GenericGCTests):
     def define_do_malloc_operations_in_call(cls):
         P = lltype.GcStruct('P', ('x', lltype.Signed))
         def g():
-            llop.do_malloc_fixedsize_clear(llmemory.GCREF)  # placeholder
+            llop.do_malloc_fixedsize(llmemory.GCREF)  # placeholder
         def f():
             q = lltype.malloc(P)
             q.x = 1
@@ -827,13 +902,13 @@ class GenericMovingGCTests(GenericGCTests):
             from rpython.translator.translator import graphof
             from rpython.flowspace.model import Constant
             from rpython.rtyper.lltypesystem import rffi
-            layoutbuilder = cls.ensure_layoutbuilder(translator)            
+            layoutbuilder = cls.ensure_layoutbuilder(translator)
             type_id = layoutbuilder.get_type_id(P)
             #
-            # now fix the do_malloc_fixedsize_clear in the graph of g
+            # now fix the do_malloc_fixedsize in the graph of g
             graph = graphof(translator, g)
             for op in graph.startblock.operations:
-                if op.opname == 'do_malloc_fixedsize_clear':
+                if op.opname == 'do_malloc_fixedsize':
                     op.args = [Constant(type_id, llgroup.HALFWORD),
                                Constant(llmemory.sizeof(P), lltype.Signed),
                                Constant(False, lltype.Bool), # has_finalizer
@@ -1116,7 +1191,8 @@ class TestGenerationGC(GenericMovingGCTests):
 
     def test_adr_of_nursery(self):
         run = self.runner("adr_of_nursery")
-        res = run([])        
+        res = run([])
+
 
 class TestGenerationalNoFullCollectGC(GCTest):
     # test that nursery is doing its job and that no full collection
@@ -1167,7 +1243,6 @@ class TestGenerationalNoFullCollectGC(GCTest):
 
 class TestHybridGC(TestGenerationGC):
     gcname = "hybrid"
-    GC_CAN_MALLOC_NONMOVABLE = True
 
     class gcpolicy(gc.BasicFrameworkGcPolicy):
         class transformerclass(shadowstack.ShadowStackFrameworkGCTransformer):
@@ -1229,10 +1304,6 @@ class TestHybridGC(TestGenerationGC):
         res = run([])
         assert res == 42
 
-    def test_malloc_nonmovable_fixsize(self):
-        py.test.skip("not supported")
-
-
 class TestMiniMarkGC(TestHybridGC):
     gcname = "minimark"
     GC_CAN_TEST_ID = True
@@ -1275,6 +1346,48 @@ class TestMiniMarkGC(TestHybridGC):
         res = run([])
         assert res == 123
 
+    def define_nursery_hash_base(cls):
+        class A:
+            pass
+        def fn():
+            objects = []
+            hashes = []
+            for i in range(200):
+                rgc.collect(0)     # nursery-only collection, if possible
+                obj = A()
+                objects.append(obj)
+                hashes.append(compute_identity_hash(obj))
+            unique = {}
+            for i in range(len(objects)):
+                assert compute_identity_hash(objects[i]) == hashes[i]
+                unique[hashes[i]] = None
+            return len(unique)
+        return fn
+
+    def test_nursery_hash_base(self):
+        res = self.runner('nursery_hash_base')
+        assert res([]) >= 195
+
+    def define_instantiate_nonmovable(cls):
+        from rpython.rlib import objectmodel
+        from rpython.rtyper import annlowlevel
+        class A:
+            pass
+        def fn():
+            a1 = A()
+            a = objectmodel.instantiate(A, nonmovable=True)
+            a.next = a1  # 'a' is known young here, so no write barrier emitted
+            res = rgc.can_move(annlowlevel.cast_instance_to_base_ptr(a))
+            rgc.collect()
+            objectmodel.keepalive_until_here(a)
+            return res
+        return fn
+
+    def test_instantiate_nonmovable(self):
+        res = self.runner('instantiate_nonmovable')
+        assert res([]) == 0
+
+
 class TestIncrementalMiniMarkGC(TestMiniMarkGC):
     gcname = "incminimark"
 
@@ -1292,6 +1405,38 @@ class TestIncrementalMiniMarkGC(TestMiniMarkGC):
                          }
             root_stack_depth = 200
 
+    def define_malloc_array_of_gcptr(self):
+        S = lltype.GcStruct('S', ('x', lltype.Signed))
+        A = lltype.GcArray(lltype.Ptr(S))
+        def f():
+            lst = lltype.malloc(A, 5)
+            return (lst[0] == lltype.nullptr(S)
+                    and lst[1] == lltype.nullptr(S)
+                    and lst[2] == lltype.nullptr(S)
+                    and lst[3] == lltype.nullptr(S)
+                    and lst[4] == lltype.nullptr(S))
+        return f
+
+    def test_malloc_array_of_gcptr(self):
+        run = self.runner('malloc_array_of_gcptr')
+        res = run([])
+        assert res
+
+    def define_malloc_struct_of_gcptr(cls):
+        S1 = lltype.GcStruct('S', ('x', lltype.Signed))
+        S = lltype.GcStruct('S',
+                                 ('x', lltype.Signed),
+                                 ('filed1', lltype.Ptr(S1)),
+                                 ('filed2', lltype.Ptr(S1)))
+        s0 = lltype.malloc(S)
+        def f():
+            return (s0.filed1 == lltype.nullptr(S1) and s0.filed2 == lltype.nullptr(S1))
+        return f
+
+    def test_malloc_struct_of_gcptr(self):
+        run = self.runner("malloc_struct_of_gcptr")
+        res = run([])
+        assert res
 
 # ________________________________________________________________
 # tagged pointers
@@ -1349,6 +1494,22 @@ class TaggedPointerGCTests(GCTest):
         res = func([])
         assert res == -1999
 
+    def define_gettypeid(cls):
+        class A(object):
+            pass
+
+        def fn():
+            a = A()
+            return rgc.get_typeid(a)
+
+        return fn
+
+    def test_gettypeid(self):
+        func = self.runner("gettypeid")
+        res = func([])
+        print res
+
+
 from rpython.rlib.objectmodel import UnboxedValue
 
 class TaggedBase(object):
@@ -1380,3 +1541,6 @@ class TestHybridTaggedPointerGC(TaggedPointerGCTests):
                          'nursery_size': 32*WORD,
                          'translated_to_c': False}
             root_stack_depth = 200
+
+    def test_gettypeid(self):
+        py.test.skip("fails for obscure reasons")
