@@ -3,7 +3,7 @@ from rpython.rlib import rfloat
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from rpython.translator import cdir
 from rpython.rtyper.lltypesystem import lltype, rffi
-from rpython.rlib import jit, objectmodel
+from rpython.rlib import jit
 from rpython.rlib.rstring import StringBuilder
 import py, sys
 
@@ -39,7 +39,7 @@ eci = ExternalCompilationInfo(
 _INT_LIMIT = 0x7ffff000
 
 dg_strtod = rffi.llexternal(
-    '_PyPy_dg_strtod', [rffi.CONST_CCHARP, rffi.CCHARPP], rffi.DOUBLE,
+    '_PyPy_dg_strtod', [rffi.CCHARP, rffi.CCHARPP], rffi.DOUBLE,
     compilation_info=eci, sandboxsafe=True)
 
 dg_dtoa = rffi.llexternal(
@@ -54,20 +54,18 @@ dg_freedtoa = rffi.llexternal(
 def strtod(input):
     if len(input) > _INT_LIMIT:
         raise MemoryError
-    if objectmodel.revdb_flag_io_disabled():
-        return _revdb_strtod(input)
     end_ptr = lltype.malloc(rffi.CCHARPP.TO, 1, flavor='raw')
     try:
         # note: don't use the class scoped_view_charp here, it
         # break some tests because this function is used by the GC
-        ll_input, llobj, flag = rffi.get_nonmovingbuffer_ll_final_null(input)
+        ll_input, flag = rffi.get_nonmovingbuffer_final_null(input)
         try:
-            result = dg_strtod(rffi.cast(rffi.CONST_CCHARP, ll_input), end_ptr)
+            result = dg_strtod(ll_input, end_ptr)
 
             endpos = (rffi.cast(lltype.Signed, end_ptr[0]) -
                       rffi.cast(lltype.Signed, ll_input))
         finally:
-            rffi.free_nonmovingbuffer_ll(ll_input, llobj, flag)
+            rffi.free_nonmovingbuffer(input, ll_input, flag)
     finally:
         lltype.free(end_ptr, flavor='raw')
 
@@ -240,8 +238,6 @@ def dtoa(value, code='r', mode=0, precision=0, flags=0,
          special_strings=lower_special_strings, upper=False):
     if precision > _INT_LIMIT:
         raise MemoryError
-    if objectmodel.revdb_flag_io_disabled():
-        return _revdb_dtoa(value)
     decpt_ptr = lltype.malloc(rffi.INTP.TO, 1, flavor='raw')
     try:
         sign_ptr = lltype.malloc(rffi.INTP.TO, 1, flavor='raw')
@@ -305,13 +301,3 @@ def dtoa_formatd(value, code, precision, flags):
 
     return dtoa(value, code, mode=mode, precision=precision, flags=flags,
                 special_strings=special_strings, upper=upper)
-
-def _revdb_strtod(input):
-    # moved in its own function for the import statement
-    from rpython.rlib import revdb
-    return revdb.emulate_strtod(input)
-
-def _revdb_dtoa(value):
-    # moved in its own function for the import statement
-    from rpython.rlib import revdb
-    return revdb.emulate_dtoa(value)
